@@ -49,11 +49,11 @@
 namespace
 {
 
-const char* RECENT_DUMMY_TEXT = N_("No recently used documents available.");
-const char* NETWORK_DUMMY_TEXT = N_("No sessions found on the local network.");
+const char RECENT_DUMMY_TEXT[] = N_("No recently used documents available.");
+const char NETWORK_DUMMY_TEXT[] = N_("No sessions found on the local network.");
 
 #ifndef GLOM_ENABLE_CLIENT_ONLY
-const char* TEMPLATE_DUMMY_TEXT = N_("No templates available.");
+const char TEMPLATE_DUMMY_TEXT[] = N_("No templates available.");
 #endif
 
 //TODO_Performance: A DomParser or XmlReader might be faster, or even a regex.
@@ -101,7 +101,7 @@ private:
 namespace Glom
 {
 
-const char* Dialog_ExistingOrNew::glade_id("dialog_existing_or_new");
+const char Dialog_ExistingOrNew::glade_id[] = "dialog_existing_or_new";
 const bool Dialog_ExistingOrNew::glade_developer(false);
 
 Dialog_ExistingOrNew::Dialog_ExistingOrNew(BaseObjectType* cobject, const Glib::RefPtr<Gtk::Builder>& builder)
@@ -166,7 +166,7 @@ Dialog_ExistingOrNew::Dialog_ExistingOrNew(BaseObjectType* cobject, const Glib::
   if(!Glib::file_test(path, Glib::FILE_TEST_EXISTS))
     path = GLOM_DOCDIR_EXAMPLES;
 #else
-  const char *const path = GLOM_DOCDIR_EXAMPLES;
+  const char path[] = GLOM_DOCDIR_EXAMPLES;
 #endif //G_OS_WIN32
 
   //Show the installed example files,
@@ -962,7 +962,6 @@ void Dialog_ExistingOrNew::on_select_clicked()
     Hildon::FileChooserDialog dialog(Gtk::FILE_CHOOSER_ACTION_OPEN);
     #else
     Gtk::FileChooserDialog dialog(*this, "Open Glom File");
-    dialog.set_icon_name("glom");
     dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
     dialog.add_button(Gtk::Stock::OPEN, Gtk::RESPONSE_OK);
     dialog.set_default_response(Gtk::RESPONSE_OK);
