@@ -21,15 +21,16 @@
 #ifndef GLOM_CONNECTIONPOOL_H
 #define GLOM_CONNECTIONPOOL_H
 
+#include <libglom/libglom_config.h>
+
 #include <libgdamm.h>
-#include <glom/libglom/sharedptr.h>
-#include <glom/libglom/data_structure/fieldtypes.h>
-#include <glom/libglom/data_structure/field.h>
+#include <libglom/sharedptr.h>
+#include <libglom/data_structure/fieldtypes.h>
+#include <libglom/data_structure/field.h>
+#include <libglom/connectionpool_backends/backend.h>
 
 #include <memory> // For std::auto_ptr
 
-#include "config.h" // For GLOM_ENABLE_CLIENT_ONLY
-
 //Avoid including the header here:
 extern "C"
 {
@@ -40,7 +41,6 @@ typedef struct _EpcAuthContext EpcAuthContext;
 
 namespace Gtk
 {
-  class Window;
   class Dialog;
 }
 
@@ -49,26 +49,6 @@ namespace Glom
 
 class AvahiPublisher;
 
-class ExceptionConnection : public std::exception
-{
-public:
-  enum failure_type
-  {
-    FAILURE_NO_SERVER, //Either there was no attempt to connect to a specific database, or the connection failed both with and without specifying the database.
-    FAILURE_NO_DATABASE //Connection without specifying the database was possible.
-  };
-
-  ExceptionConnection(failure_type failure);
-  virtual ~ExceptionConnection() throw();
-
-  virtual const char* what() const throw();
-
-  virtual failure_type get_failure_type() const;
-
-protected:
-  failure_type m_failure_type;
-};
-
 /** When the SharedConnection is destroyed, it will inform the connection pool,
  * so that the connection pool can keep track of who is using the connection,
  * so that it can close it when nobody is using it.
@@ -87,122 +67,67 @@ public:
    */
   void close();
 
+  //TODO: Document this:
   typedef sigc::signal<void> type_signal_finished;
   type_signal_finished signal_finished();
 
-protected:
+private:
   Glib::RefPtr<Gnome::Gda::Connection> m_gda_connection;
 
   type_signal_finished m_signal_finished;
 };
 
-class Document_Glom;
-
-//TODO: Put this in the sub-directory:
-
-/** This hides database specific functionality from the ConnectionPool, so
- * the ConnectionPool can be used without worrying about the actual database
- * backend in use. Use ConnectionPool::set_backend() to set the backend for
- * the connectionpool to use. */
-class ConnectionPoolBackend
-{
-  friend class ConnectionPool;
-public:
-  virtual ~ConnectionPoolBackend() {}
-  typedef std::vector<sharedptr<const Field> > type_vecConstFields;
-
-protected:
-  /** Helper functions for backend implementations to use, so that these don't
-   * need to worry whether glibmm was compiled with exceptions or not.
-   */
-  bool query_execute(const Glib::RefPtr<Gnome::Gda::Connection>& connection, const Glib::ustring& sql_query, std::auto_ptr<Glib::Error>& error);
-  bool set_server_operation_value(const Glib::RefPtr<Gnome::Gda::ServerOperation>& operation, const Glib::ustring& path, const Glib::ustring& value, std::auto_ptr<Glib::Error>& error);
-  Glib::RefPtr<Gnome::Gda::ServerOperation> create_server_operation(const Glib::RefPtr<Gnome::Gda::ServerProvider>& provider, const Glib::RefPtr<Gnome::Gda::Connection>& connection, Gnome::Gda::ServerOperationType type, std::auto_ptr<Glib::Error>& error);
-  bool perform_server_operation(const Glib::RefPtr<Gnome::Gda::ServerProvider>& provider, const Glib::RefPtr<Gnome::Gda::Connection>& connection, const Glib::RefPtr<Gnome::Gda::ServerOperation>& operation, std::auto_ptr<Glib::Error>& error);
-  bool begin_transaction(const Glib::RefPtr<Gnome::Gda::Connection>& connection, const Glib::ustring& name, Gnome::Gda::TransactionIsolation level, std::auto_ptr<Glib::Error>& error);
-  bool commit_transaction(const Glib::RefPtr<Gnome::Gda::Connection>& connection, const Glib::ustring& name, std::auto_ptr<Glib::Error>& error);
-  bool rollback_transaction(const Glib::RefPtr<Gnome::Gda::Connection>& connection, const Glib::ustring& name, std::auto_ptr<Glib::Error>& error);
-
-  /* TODO: Merge create_database() and initialize() into a single function?
-   */
-
-  /** This method specifies the format of values in SQL expressions.
-   */
-  virtual Field::sql_format get_sql_format() const = 0;
-
-  /* Whether the database can be accessed from remote machines, once startup()
-   * was called.
-   */
-  virtual bool supports_remote_access() const = 0;
-
-  /** This method is called for one-time initialization of the database
-   * storage. No need to implement this function if the data is centrally
-   * hosted, not managed by Glom.
-   */
-  virtual bool initialize(Gtk::Window* parent_window, const Glib::ustring& initial_username, const Glib::ustring& password);
-
-  /** This method is called before the backend is used otherwise. This can
-   * be used to start a self-hosted database server. There is no need to implement
-   * this function if there is no need for extra startup code.
-   */
-  virtual bool startup(Gtk::Window* parent_window);
-
-  /** This method is called when the backend is no longer used. This can be
-   * used to shut down a self-hosted database server. There is no need to
-   * implement this function if there is no need for extra cleanup code.
-   */
-  virtual void cleanup(Gtk::Window* parent_window);
-
-  /** This method is called to create a connection to the database server.
-   * There exists only the variant with an error variable as last parameter
-   * so we don't need #ifdefs all over the code. This part of the API is only
-   * used by the ConnectionPool which will translate the error back into
-   * an exception in case exceptions are enabled.
-    */
-  virtual Glib::RefPtr<Gnome::Gda::Connection> connect(const Glib::ustring& database, const Glib::ustring& username, const Glib::ustring& password, std::auto_ptr<ExceptionConnection>& error) = 0;
-
-#ifndef GLOM_ENABLE_CLIENT_ONLY
-  virtual bool add_column(const Glib::RefPtr<Gnome::Gda::Connection>& connection, const Glib::ustring& table_name, const sharedptr<const Field>& field, std::auto_ptr<Glib::Error>& error);
-
-  virtual bool drop_column(const Glib::RefPtr<Gnome::Gda::Connection>& connection, const Glib::ustring& table_name, const Glib::ustring& field_name, std::auto_ptr<Glib::Error>& error);
-
-  virtual bool change_columns(const Glib::RefPtr<Gnome::Gda::Connection>& connection, const Glib::ustring& table_name, const type_vecConstFields& old_fields, const type_vecConstFields& new_fields, std::auto_ptr<Glib::Error>& error);
-
-  /** This method is called to create a new database on the
-   * database server. */
-  virtual bool create_database(const Glib::ustring& database_name, const Glib::ustring& username, const Glib::ustring& password, std::auto_ptr<Glib::Error>& error) = 0;
-#endif
-};
+class Document;
 
 /** This is a singleton.
  * Use get_instance().
  */
 class ConnectionPool : public sigc::trackable
 {
-protected:
+private:
   ConnectionPool();
   //ConnectionPool(const ConnectionPool& src);
   virtual ~ConnectionPool();
   //ConnectionPool& operator=(const ConnectionPool& src);
 
 public:
-  typedef ConnectionPoolBackend::type_vecConstFields type_vecConstFields;
+  typedef ConnectionPoolBackends::Backend Backend;
+  typedef Backend::type_vec_const_fields type_vec_const_fields;
 
   /** Get the singleton instance.
    * Use delete_instance() when the program quits.
    */
   static ConnectionPool* get_instance();
 
+  /** Make the ConnectionPool use the correct backend, with the necessary details,
+   * as required by the document.
+   */
+  void setup_from_document(const Document* document);
+
   /// Delete the singleton so it doesn't show up as leaked memory in, for instance, valgrind.
   static void delete_instance();
 
+  typedef sigc::slot<void> type_void_slot;
+
+#ifndef G_OS_WIN32
+  /** Set callbacks that will be called to show UI while starting to advertise
+   * on the network via Avahi.
+   *
+   * @param slot_begin Show an explanatory message.
+   * @param slot_progress Show a pulse progress and/or keep the UI updating.
+   * @param slot_done Stop showing the message.
+   */
+  void set_avahi_publish_callbacks(const type_void_slot& slot_begin, const type_void_slot& slot_progress, const type_void_slot& slot_done);
+#endif
+
+
   bool get_ready_to_connect() const;
   void set_ready_to_connect(bool val = true);
 
-  void set_backend(std::auto_ptr<ConnectionPoolBackend> backend);
+  void set_backend(std::auto_ptr<Backend> backend);
 
-  ConnectionPoolBackend* get_backend();
-  const ConnectionPoolBackend* get_backend() const;
+  Backend* get_backend();
+  const Backend* get_backend() const;
 
   /** This method will return a SharedConnection, either by opening a new connection or returning an already-open connection.
    * When that SharedConnection is destroyed, or when SharedConnection::close() is called, then the ConnectionPool will be informed.
@@ -224,6 +149,12 @@ public:
   static sharedptr<SharedConnection> get_and_connect(std::auto_ptr<ExceptionConnection>& error);
 #endif
 
+  /** This callback should show UI to indicate that work is still happening.
+   * For instance, a pulsing ProgressBar.
+   */
+  typedef Backend::SlotProgress SlotProgress;
+
+ //TODO: Add SlotProgress?
   /** Creates a new database.
    */
 #ifdef GLIBMM_EXCEPTIONS_ENABLED
@@ -232,6 +163,24 @@ public:
   void create_database(const Glib::ustring& database_name, std::auto_ptr<Glib::Error>& error);
 #endif
 
+  /** Save a backup of the database in a tarball.
+   * This backup can later be used to recreate the database,
+   * for instance with a later version of PostgreSQL.
+   * See @convert_backup().
+   *
+   * @param path_dir The top-level directory for the backup file, using the normal directory structure.
+   *
+   */
+  bool save_backup(const SlotProgress& slot_progress, const std::string& path_dir);
+
+  /** Use a backup of the database in a tarball to create tables and data in an existing empty database.
+   * The database (server) should already have the necessary groups and users.
+   *
+   * @param path_dir The top-level directory for the backup file, using the normal directory structure.
+   * See save_backup().
+   */
+  bool convert_backup(const SlotProgress& slot_progress, const std::string& path_dir);
+
   void set_user(const Glib::ustring& value);
   void set_password(const Glib::ustring& value);
   void set_database(const Glib::ustring& value);
@@ -242,25 +191,51 @@ public:
 
   Field::sql_format get_sql_format() const;
   const FieldTypes* get_field_types() const;
+  Glib::ustring get_string_find_operator() const;
+
+  typedef Backend::InitErrors InitErrors;
 
   /** Do one-time initialization, such as  creating required database
    * files on disk for later use by their own  database server instance.
+   *
+   * @param slot_progress A callback to call while the work is still happening.
+   * @param network_shared Whether the database (and document) should be available to other users over the network,
+   * if possible.
    * @param parent_window A parent window to use as the transient window when displaying errors.
    */
-  bool initialize(Gtk::Window* parent_window);
+  InitErrors initialize(const SlotProgress& slot_progress, bool network_shared = false);
 
-  /** Start a database server instance for the exisiting database files.
+  typedef Backend::StartupErrors StartupErrors;
+
+  /** Start a database server instance for the existing database files.
+   *
+   * @param slot_progress A callback to call while the work is still happening.
+   * @param network_shared Whether the database (and document) should be available to other users over the network,
+   * if possible.
    * @param parent_window The parent window (transient for) of any dialogs shown during this operation.
    * @result Whether the operation was successful.
    */
-  bool startup(Gtk::Window* parent_window);
+  StartupErrors startup(const SlotProgress& slot_progress, bool network_shared = false);
 
   /** Stop the database server instance for the database files.
+   *
+   * @param slot_progress A callback to call while the work is still happening.
    * @param parent_window The parent window (transient for) of any dialogs shown during this operation.
    */
-  void cleanup(Gtk::Window* parent_window);
+  bool cleanup(const SlotProgress& slot_progress);
+
+  /** Change the database server's configration to allow or prevent access from
+   * other users on the network.
+   *
+   * For current backends, you may use this only before startup(),
+   * or after cleanup().
+   *
+   * @param slot_progress A callback to call while the work is still happening.
+   * @param network_shared Whether the database (and document) should be available to other users over the network,
+   * if possible.
+   */
+  virtual bool set_network_shared(const SlotProgress& slot_progress, bool network_shared = true);
 
-#ifndef GLOM_ENABLE_CLIENT_ONLY
 #ifdef GLIBMM_EXCEPTIONS_ENABLED
   bool add_column(const Glib::ustring& table_name, const sharedptr<const Field>& field);
 #else
@@ -280,17 +255,16 @@ public:
 #endif
 
 #ifdef GLIBMM_EXCEPTIONS_ENABLED
-  bool change_columns(const Glib::ustring& table_name, const type_vecConstFields& old_fields, const type_vecConstFields& fields);
+  bool change_columns(const Glib::ustring& table_name, const type_vec_const_fields& old_fields, const type_vec_const_fields& fields);
 #else
-  bool change_columns(const Glib::ustring& table_name, const type_vecConstFields& old_fields, const type_vecConstFields& fields, std::auto_ptr<Glib::Error>& error);
+  bool change_columns(const Glib::ustring& table_name, const type_vec_const_fields& old_fields, const type_vec_const_fields& fields, std::auto_ptr<Glib::Error>& error);
 #endif
-#endif // !GLOM_ENABLE_CLIENT_ONLY
 
   /** Specify a callback that the ConnectionPool can call to get a pointer to the document.
-   * This callback avoids Connection having to link to App_Glom,
+   * This callback avoids Connection having to link to Application,
    * and avoids us worrying about whether a previously-set document (via a set_document() method) is still valid.
-   */ 
-  typedef sigc::slot<Document_Glom*> SlotGetDocument; 
+   */
+  typedef sigc::slot<Document*> SlotGetDocument;
   void set_get_document_func(const SlotGetDocument& slot);
 
 #ifndef G_OS_WIN32
@@ -302,24 +276,27 @@ public:
   static void on_epc_progress_end(gpointer user_data);
 #endif // !G_OS_WIN32
 
+  //TODO: Document
+  void set_show_debug_output(bool val);
+  bool get_show_debug_output() const;
+
   //Show the gda error in a dialog.
-  static bool handle_error(bool cerr_only = false);
+  static bool handle_error_cerr_only();
 
-  /** Postgres can't be started as root. initdb complains.
-   * So just prevent this in general. It is safer anyway.
-   */
-  static bool check_user_is_not_root();
-
-protected:
+private:
   void on_sharedconnection_finished();
 
-#ifndef GLOM_ENABLE_CLIENT_ONLY
+  /** We call this when we know that the current connection will no longer work,
+   * for instance after we have stopped the server.
+   */
+  void invalidate_connection();
+
   /** Examine ports one by one, starting at @a starting_port, in increasing order,
    * and return the first one that is available.
    */
   //static int discover_first_free_port(int start_port, int end_port);
 
-  Document_Glom* get_document();
+  Document* get_document();
 
 #ifndef G_OS_WIN32
   /** Advertize self-hosting via avahi:
@@ -327,32 +304,29 @@ protected:
   void avahi_start_publishing();
   void avahi_stop_publishing();
 #endif // !G_OS_WIN32
-#endif // !GLOM_ENABLE_CLIENT_ONLY
 
-protected:
+private:
 
-#ifndef GLOM_ENABLE_CLIENT_ONLY
   EpcPublisher* m_epc_publisher;
   Gtk::Dialog* m_dialog_epc_progress; //For progress while generating certificates.
-#endif // !GLOM_ENABLE_CLIENT_ONLY
 
-  std::auto_ptr<ConnectionPoolBackend> m_backend;
+  std::auto_ptr<Backend> m_backend;
   Glib::RefPtr<Gnome::Gda::Connection> m_refGdaConnection;
   guint m_sharedconnection_refcount;
   bool m_ready_to_connect;
   Glib::ustring m_host, m_user, m_password, m_database;
 
   FieldTypes* m_pFieldTypes;
+  bool m_show_debug_output;
 
 private:
 
   static ConnectionPool* m_instance;
-#ifndef GLOM_ENABLE_CLIENT_ONLY
   SlotGetDocument m_slot_get_document;
-#endif
+
+  type_void_slot m_epc_slot_begin, m_epc_slot_progress, m_epc_slot_done;
 };
 
 } //namespace Glom
 
 #endif //GLOM_CONNECTIONPOOL_H
-
