@@ -22,9 +22,9 @@
 #define GLOM_DATASTRUCTURE_FIELD_H
 
 #include <libgdamm.h>
-#include "translatable_item.h"
-#include "layout/fieldformatting.h"
-#include <glom/libglom/sharedptr.h>
+#include <libglom/data_structure/translatable_item.h>
+#include <libglom/data_structure/layout/fieldformatting.h>
+#include <libglom/sharedptr.h>
 
 //Predicate, for use with std::find_if():
 
@@ -62,7 +62,7 @@ public:
     return (element->get_name() == m_strName);
   }
 
-protected:
+private:
   Glib::ustring m_strName;
 };
 
@@ -165,18 +165,31 @@ public:
    */
   Glib::ustring get_gda_holder_string(const Glib::ustring& name = Glib::ustring()) const;
 
-  /** Escape the value so that it can be used in a SQL command.
+  /** Escape and quote the value so that it can be used in a SQL command.
    */
-  Glib::ustring sql(const Gnome::Gda::Value& value, sql_format format) const;
+  Glib::ustring sql(const Gnome::Gda::Value& value, const Glib::RefPtr<Gnome::Gda::Connection>& connection) const;
 
-  /** Escape the value so that it can be used in a SQL command.
+  //TODO: Remove this:
+  /** Escape and quote the value so that it can be used in a SQL command.
    * Uses the sql_format of the current connectionpool backend.
    */
   Glib::ustring sql(const Gnome::Gda::Value& value) const;
 
-  /** Unescape the value again.
+  /** Get the canonical format for a file, for instance for 
+   * a default value or for example data.
+   * This does not add quotes for text fields so the caller may need to do that.
+   * Note that this does not do any extra escaping such as an XML file might need.
    */
-  Gnome::Gda::Value from_sql(const Glib::ustring& str, sql_format format, bool& success) const;
+  Glib::ustring to_file_format(const Gnome::Gda::Value& value) const;
+
+  static Glib::ustring to_file_format(const Gnome::Gda::Value& value, glom_field_type glom_type);
+
+  /** Parse the value from the canonical file format. See to_file_format()
+   * This does note remove quotes from text values so the caller may need to do that.
+   */
+  Gnome::Gda::Value from_file_format(const Glib::ustring& str, bool& success) const;
+
+  static Gnome::Gda::Value from_file_format(const Glib::ustring& str, glom_field_type glom_type, bool& success);
 
   /** Escape the value so that it can be used in a SQL command for a find.
    */
@@ -224,7 +237,7 @@ public:
 
   FieldFormatting m_default_formatting;
 
-protected:
+private:
 
   static void init_map();
 
