@@ -81,6 +81,9 @@ public:
 
   bool get_is_example_file() const;
   void set_is_example_file(bool value = true);
+  
+  bool get_is_backup_file() const;
+  void set_is_backup_file(bool value = true);
 
   /* Get version of the document format used for this document.
    *  This can increase when the file has been re-saved.
@@ -124,7 +127,7 @@ public:
 
   void set_connection_server(const Glib::ustring& strVal);
   void set_connection_database(const Glib::ustring& strVal);
-  void set_connection_port(int port_number);
+  void set_connection_port(unsigned int port_number);
   void set_connection_try_other_ports(bool val);
 
  /** Temporarily set a username in the document.
@@ -144,7 +147,7 @@ public:
 
   Glib::ustring get_connection_server() const;
   Glib::ustring get_connection_database() const;
-  int get_connection_port() const;
+  unsigned int get_connection_port() const;
   bool get_connection_try_other_ports() const;
 
   /** Retrieve a username previously set in the document.
@@ -182,7 +185,7 @@ public:
   /** Returns whether the field is the from-field in a to-one relationship.
    * @see get_relationship_is_to_one(). Ignores hidden tables.
    */
-  sharedptr<Relationship> get_field_used_in_relationship_to_one(const Glib::ustring& table_name, const sharedptr<const LayoutItem_Field>& layout_field) const;
+  sharedptr<const Relationship> get_field_used_in_relationship_to_one(const Glib::ustring& table_name, const sharedptr<const LayoutItem_Field>& layout_field) const;
 
   typedef std::vector< sharedptr<Field> > type_vec_fields;
   type_vec_fields get_table_fields(const Glib::ustring& table_name) const;
@@ -511,7 +514,7 @@ private:
 
   Glib::ustring m_connection_server, m_connection_database;
   Glib::ustring m_connection_user; //Don't save the user.
-  int m_connection_port; //0 means any port. Ignored when self-hosting (which may use a different port each time).
+  unsigned int m_connection_port; //0 means any port. Ignored when self-hosting (which may use a different port each time).
   bool m_connection_try_other_ports; //Set to false for self-hosted or browsed-from-network documents.
 
   class LayoutInfo
@@ -618,6 +621,7 @@ private:
   bool m_block_modified_set;
   bool m_allow_auto_save;
   bool m_is_example;
+  bool m_is_backup;
   guint m_document_format_version;
 
   bool m_opened_from_browse;
