@@ -19,19 +19,24 @@
  */
 
 #include <libglom/spawn_with_feedback.h>
-#include <libglom/dialog_progress_creating.h>
-#include <libglom/glade_utils.h>
-#include <bakery/bakery.h>
+#include <glibmm/main.h>
+#include <glibmm/spawn.h>
+#include <glibmm/thread.h>
+#include <glibmm/iochannel.h>
+#include <glibmm/shell.h>
+#include <glibmm/fileutils.h>
+#include <glibmm/miscutils.h>
 #include <glibmm/i18n.h>
+#include <memory> //For auto_ptr.
+#include <stdexcept>
 #include <iostream>
 
 #ifdef G_OS_WIN32
-#define SAVE_DATADIR DATADIR
-#undef DATADIR
-#include <windows.h>
-#define DATADIR SAVE_DATADIR
+# include <windows.h>
 #endif
 
+// Uncomment to see debug messages
+//#define GLOM_SPAWN_DEBUG
 
 namespace Glom
 {
@@ -48,9 +53,17 @@ public:
   bool* m_result;
 };
 
-// This is a simple process launching API wrapping g_spawn_async on linux and
-// CreateProcess() on Windows. We need to use CreateProcess on Windows to be
+static void on_spawn_info_finished(const Glib::RefPtr<Glib::MainLoop>& mainloop)
+{
+  //Allow our mainloop.run() to return:
+  if(mainloop)
+    mainloop->quit();
+}
+
+// This is a simple-process launching API wrapping g_spawn_async on linux and
+// CreateProcess() on Windows. We need to use CreateProcess() on Windows to be
 // able to suppress the console window.
+// TODO: File a bug about the console window on Windows.
 namespace Impl
 {
 
@@ -61,7 +74,9 @@ static const unsigned int REDIRECT_STDERR = 2;
 class SpawnError: public std::runtime_error
 {
 public:
-  SpawnError(const std::string& error_message): std::runtime_error(error_message) {}
+  explicit SpawnError(const std::string& error_message)
+  : std::runtime_error(error_message)
+  {}
 };
 #endif
 
@@ -92,10 +107,10 @@ private:
     // it can write to it.
     SECURITY_ATTRIBUTES security_attr;
     security_attr.nLength = sizeof(security_attr);
-    security_attr.lpSecurityDescriptor = NULL;
-    security_attr.bInheritHandle = TRUE;
+    security_attr.lpSecurityDescriptor = 0;
+    security_attr.bInheritHandle = true;
 
-    HANDLE result = CreateFile(filename.c_str(), GENERIC_WRITE, FILE_SHARE_READ | FILE_SHARE_DELETE, &security_attr, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
+    HANDLE result = CreateFile(filename.c_str(), GENERIC_WRITE, FILE_SHARE_READ | FILE_SHARE_DELETE, &security_attr, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, 0);
 #ifdef GLIBMM_EXCEPTIONS_ENABLED
     if(!result) throw SpawnError(win32_error_message());
 #endif // GLIBMM_EXCEPTIONS_ENABLED
@@ -116,7 +131,7 @@ private:
       char buffer[1024 + 1];
       gsize bytes_read;
 
-      Glib::IOStatus status;
+      Glib::IOStatus status = Glib::IO_STATUS_NORMAL;
 #ifdef GLIBMM_EXCEPTIONS_ENABLED
       try
       {
@@ -154,7 +169,13 @@ private:
     channel->set_flags(Glib::IO_FLAG_NONBLOCK, error);
 #endif // !GLIBMM_EXCEPTIONS_ENABLED
 
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
     channel->set_encoding("");
+#else
+    channel->set_encoding("", error);
+    if (error.get())
+      std::cerr << "Error: " << error->what() << std::endl;
+#endif
     channel->set_buffered(false);
 
     Glib::signal_io().connect(sigc::bind(sigc::mem_fun(*this, &SpawnInfo::on_io), channel, sigc::ref(string)), channel, Glib::IO_IN);
@@ -164,17 +185,19 @@ private:
 public:
   typedef sigc::signal<void> SignalFinished;
 
+  /** TODO: Document the redirect parameter.
+   */
   SpawnInfo(const Glib::ustring& command_line, int redirect):
     running(false), return_status(0)
   {
 #ifdef G_OS_WIN32
     startup_info.cb = sizeof(startup_info);
     startup_info.dwFlags = STARTF_USESTDHANDLES;
-    startup_info.lpReserved = NULL;
-    startup_info.lpDesktop = NULL;
-    startup_info.lpTitle = NULL;
+    startup_info.lpReserved = 0;
+    startup_info.lpDesktop = 0;
+    startup_info.lpTitle = 0;
     startup_info.cbReserved2 = 0;
-    startup_info.lpReserved2 = NULL;
+    startup_info.lpReserved2 = 0;
     startup_info.hStdInput = INVALID_HANDLE_VALUE;
 
     if(redirect & REDIRECT_STDOUT)
@@ -192,7 +215,7 @@ public:
     std::copy(command_line.data(), command_line.data() + command_line.length(), command.begin());
     command[command_line.length()] = '\0';
 
-    if(!CreateProcess(NULL, &command[0], NULL, NULL, TRUE, CREATE_NO_WINDOW, NULL, NULL, &startup_info, &process_info))
+    if(!CreateProcess(0, &command[0], 0, 0, true, CREATE_NO_WINDOW, 0, 0, &startup_info, &process_info))
     {
 #ifdef GLIBMM_EXCEPTIONS_ENABLED
       throw SpawnError(win32_error_message());
@@ -206,15 +229,21 @@ public:
       std::vector<std::string> arguments = Glib::shell_parse_argv(command_line);
       int child_stdout;
       int child_stderr;
-      Glib::spawn_async_with_pipes(Glib::get_current_dir(), arguments, Glib::SPAWN_DO_NOT_REAP_CHILD, sigc::slot<void>(), &pid, NULL, redirect & REDIRECT_STDOUT ? &child_stdout : NULL, redirect & REDIRECT_STDERR ? &child_stderr : NULL);
-
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
+      Glib::spawn_async_with_pipes(Glib::get_current_dir(), arguments, Glib::SPAWN_DO_NOT_REAP_CHILD, sigc::slot<void>(), &pid, 0, redirect & REDIRECT_STDOUT ? &child_stdout : 0, redirect & REDIRECT_STDERR ? &child_stderr : 0);
+#else
+      std::auto_ptr<Glib::Error> error;
+      Glib::spawn_async_with_pipes(Glib::get_current_dir(), arguments, Glib::SPAWN_DO_NOT_REAP_CHILD, sigc::slot<void>(), &pid, 0, redirect & REDIRECT_STDOUT ? &child_stdout : 0, redirect & REDIRECT_STDERR ? &child_stderr : 0, error);
+      if (error.get())
+        std::cerr << "Spawn Error: " << error->what() << std::endl;
+#endif
       if(redirect & REDIRECT_STDOUT)
         redirect_to_string(child_stdout, stdout_text);
       if(redirect & REDIRECT_STDERR)
         redirect_to_string(child_stderr, stderr_text);
     }
 #ifdef GLIBMM_EXCEPTIONS_ENABLED
-    catch(Glib::Exception& ex)
+    catch(const Glib::Exception& ex)
     {
       throw SpawnError(ex.what());
     }
@@ -309,169 +338,232 @@ private:
 #endif
 };
 
-std::auto_ptr<const SpawnInfo> spawn_async(const Glib::ustring& command_line, int redirect)
+static std::auto_ptr<const SpawnInfo> spawn_async(const Glib::ustring& command_line, int redirect)
 {
   return std::auto_ptr<const SpawnInfo>(new SpawnInfo(command_line, redirect));
 }
 
-bool spawn_async_end(std::auto_ptr<const SpawnInfo> info, std::string* stdout_text = NULL, std::string* stderr_text = NULL, int* return_status = NULL)
+/**
+ * @param return_status: The return value of the command.
+ * @result Whether we successfully ended the async spawn.
+ */
+static bool spawn_async_end(std::auto_ptr<const SpawnInfo> info, std::string* stdout_text = 0, std::string* stderr_text = 0, int* return_status = 0)
 {
-  if(stdout_text) info->get_stdout(*stdout_text);
-  if(stderr_text) info->get_stderr(*stderr_text);
-  if(return_status) *return_status = info->get_return_status();
+  if(stdout_text)
+    info->get_stdout(*stdout_text);
+
+  if(stderr_text)
+    info->get_stderr(*stderr_text);
+
+  if(return_status)
+    *return_status = info->get_return_status();
+
   return !info->is_running();
 }
 
-int spawn_sync(const Glib::ustring& command_line, std::string* stdout_text, std::string* stderr_text)
+static int spawn_sync(const Glib::ustring& command_line, std::string* stdout_text, std::string* stderr_text)
 {
   int redirect_flags = 0;
-  if(stdout_text) redirect_flags |= REDIRECT_STDOUT;
-  if(stderr_text) redirect_flags |= REDIRECT_STDERR;
+  if(stdout_text)
+    redirect_flags |= REDIRECT_STDOUT;
 
-  std::auto_ptr<const SpawnInfo> info = spawn_async(command_line, redirect_flags);
-  info->signal_finished().connect(sigc::ptr_fun(&Gtk::Main::quit));
+  if(stderr_text)
+    redirect_flags |= REDIRECT_STDERR;
 
-  // Wait for termination
-  Gtk::Main::run();
+  Glib::RefPtr<Glib::MainLoop> mainloop = Glib::MainLoop::create(false);
+
+  std::auto_ptr<const SpawnInfo> info = spawn_async(command_line, redirect_flags); //This could throw
+  info->signal_finished().connect(
+    sigc::bind(sigc::ptr_fun(&on_spawn_info_finished), sigc::ref(mainloop) ) );
+
+  // Block until signal_finished is emitted:
+  mainloop->run();
 
   int return_status = 0;
-  bool returned = spawn_async_end(info, stdout_text, stderr_text, &return_status);
+  const bool returned = spawn_async_end(info, stdout_text, stderr_text, &return_status);
   g_assert(returned);
   return return_status;
 }
 
 } // namespace Impl
 
-static Dialog_ProgressCreating* get_and_show_pulse_dialog(const Glib::ustring& message, Gtk::Window* parent_window)
+bool execute_command_line_and_wait(const std::string& command, const SlotProgress& slot_progress)
 {
-  if(!parent_window)
-    std::cerr << "debug: Glom: get_and_show_pulse_dialog(): parent_window is NULL" << std::endl;
+  //Show UI progress feedback while we wait for the command to finish:
 
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
-  Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(Utils::get_glade_file_path("glom.glade"), "window_progress");
-#else
-  std::auto_ptr<Gnome::Glade::XmlError> error;
-  Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(Utils::get_glade_file_path("glom.glade"), "window_progress", "", error);
-  if(error.get()) return NULL;
-#endif
-
-  if(refXml)
+  std::auto_ptr<const Impl::SpawnInfo> info;
+  
+  try
   {
-    Dialog_ProgressCreating* dialog_progress = 0;
-    refXml->get_widget_derived("window_progress", dialog_progress);
-    if(dialog_progress)
-    {
-      dialog_progress->set_message(_("Processing"), message);
-      dialog_progress->set_modal();
-
-      if(parent_window)
-        dialog_progress->set_transient_for(*parent_window);
-
-      dialog_progress->show();
-
-      return dialog_progress;
-    }
+    info = Impl::spawn_async(command, 0);
   }
+  catch(const Impl::SpawnError& ex)
+  {
+    std::cerr << G_STRFUNC << ": exception: " << ex.what() << std::endl;
+    return false;
+  }
+  
+  Glib::RefPtr<Glib::MainLoop> mainloop = Glib::MainLoop::create(false);
+  info->signal_finished().connect(
+    sigc::bind(sigc::ptr_fun(&on_spawn_info_finished), sigc::ref(mainloop) ) );
 
-  return NULL;
+  // Pulse two times a second:
+  sigc::connection timeout_connection = Glib::signal_timeout().connect(
+    sigc::bind_return( slot_progress, true),
+    500);
+  slot_progress(); //Make sure it is called at least once.
+
+  //Block until signal_finished is called.
+  mainloop->run();
+
+  //Stop the timeout callback:
+  timeout_connection.disconnect();
+
+  int return_status = false;
+  const bool returned = Impl::spawn_async_end(info, 0, 0, &return_status);
+  if(!returned)
+    return false; // User closed the dialog prematurely?
+
+  return (return_status == EXIT_SUCCESS);
 }
 
-
-
-bool execute_command_line_and_wait(const std::string& command, const Glib::ustring& message, Gtk::Window* parent_window)
+bool execute_command_line_and_wait(const std::string& command, const SlotProgress& slot_progress, std::string& output)
 {
-  if(!parent_window)
-    std::cerr << "debug: Glom: execute_command_line_and_wait(): parent_window is NULL" << std::endl;
+  //Initialize output parameter:
+  output = std::string();
 
-  //Show a dialog with a pulsing progress bar and a human-readable message, while we wait for the command to finish:
-  //
-  //Put the dialog in an auto_ptr so that it will be deleted (and hidden) when the current function returns.
-  Dialog_ProgressCreating* dialog_temp = get_and_show_pulse_dialog(message, parent_window);
-  std::auto_ptr<Dialog_ProgressCreating> dialog_progress;
-  dialog_progress.reset(dialog_temp);
+  //Show UI progress feedback while we wait for the command to finish:
 
-  std::auto_ptr<const Impl::SpawnInfo> info = Impl::spawn_async(command, 0);
+  std::auto_ptr<const Impl::SpawnInfo> info;
+  
+  try
+  {
+    info = Impl::spawn_async(command, Impl::REDIRECT_STDOUT | Impl::REDIRECT_STDERR);
+  }
+  catch(const Impl::SpawnError& ex)
+  {
+    std::cerr << G_STRFUNC << ": exception: " << ex.what() << std::endl;
+    return false;
+  }
+  
+  Glib::RefPtr<Glib::MainLoop> mainloop = Glib::MainLoop::create(false);
   info->signal_finished().connect(
-    sigc::bind(sigc::mem_fun(*dialog_progress, &Dialog_ProgressCreating::response), Gtk::RESPONSE_ACCEPT));
+    sigc::bind(sigc::ptr_fun(&on_spawn_info_finished), sigc::ref(mainloop) ) );
 
-  // Pulse two times a second
-  Glib::signal_timeout().connect(
-    sigc::bind_return(sigc::mem_fun(*dialog_progress, &Dialog_ProgressCreating::pulse), true),
+  // Pulse two times a second:
+  sigc::connection timeout_connection = Glib::signal_timeout().connect(
+    sigc::bind_return( slot_progress, true),
     500);
+  slot_progress(); //Make sure it is called at least once.
 
-  dialog_progress->run();
+  //Block until signal_finished is called.
+  mainloop->run();
 
-  int return_status;
-  bool returned = Impl::spawn_async_end(info, NULL, NULL, &return_status);
-  if(!returned) return false; // User closed the dialog prematurely?
+  //Stop the timeout callback:
+  timeout_connection.disconnect();
 
-  return (return_status == 0);
+  int return_status = false;
+  std::string stdout_text, stderr_text;
+  const bool returned = Impl::spawn_async_end(info, &stdout_text, &stderr_text, &return_status);
+  if(!returned)
+    return false; // User closed the dialog prematurely?
+
+  //std::cout << "DEBUG: command=" << command << std::endl;
+  //std::cout << "  DEBUG: stdout_text=" << stdout_text << std::endl;
+  //std::cout << "  DEBUG: stderr_text=" << stderr_text << std::endl;
+
+  output = stdout_text;
+
+  if(!stderr_text.empty())
+  {
+    std::cerr << "Glom: execute_command_line_and_wait(): command produced stderr text: " << std::endl <<
+      "  command: " << command << std::endl <<
+      "  error text: " << stderr_text << std::endl;
+  }
+
+  return (return_status == EXIT_SUCCESS);
 }
 
 // Callback handlers for execute_command_line_and_wait_until_second_command_returns_success
-namespace {
-  bool on_timeout(const std::string& second_command, const std::string& success_text, Dialog_ProgressCreating* dialog_progress)
+namespace
+{
+
+  bool second_command_on_timeout(const std::string& second_command, const std::string& success_text, const SlotProgress& slot_progress, const Glib::RefPtr<Glib::MainLoop>& mainloop)
   {
     Glib::ustring stored_env_lang;
     Glib::ustring stored_env_language;
     if(!success_text.empty())
     {
-      // If we are going to check the text output of the second command, 
+      // If we are going to check the text output of the second command,
       // then we should make sure that we get a fairly canonical version of that text,
       // so we set the LANG for this command.
-      // We have to set LANGUAGE (a GNU extension) as well as LANG, because it 
-      // is probably defined on the system already and that definition would override our LANG:  
+      // We have to set LANGUAGE (a GNU extension) as well as LANG, because it
+      // is probably defined on the system already and that definition would override our LANG:
       // (Note that we can not just do "LANG=C;the_command", as on the command line, because g_spawn() does not support that.)
 
+      #ifdef GLOM_SPAWN_DEBUG
       std::cout << std::endl << "debug: temporarily setting LANG and LANGUAGE environment variables to \"C\"" << std::endl;
+      #endif //GLOM_SPAWN_DEBUG
+
       stored_env_lang = Glib::getenv("LANG");
       stored_env_language = Glib::getenv("LANGUAGE");
       Glib::setenv("LANG", "C", true /* overwrite */);
       Glib::setenv("LANGUAGE", "C", true /* overwrite */);
     }
 
+    #ifdef GLOM_SPAWN_DEBUG
     std::cout << std::endl << "debug: command_line (second): " << second_command << std::endl << std::endl;
+    #endif //GLOM_SPAWN_DEBUG
 
     int return_status = 0;
     std::string stdout_output;
 #ifdef GLIBMM_EXCEPTIONS_ENABLED
     try
     {
-      return_status = Impl::spawn_sync(second_command, &stdout_output, NULL);
+      return_status = Impl::spawn_sync(second_command, &stdout_output, 0);
     }
     catch(const Impl::SpawnError& ex)
     {
-      std::cerr << "Glom::execute_command_line_and_wait_until_second_command_returns_success() Exception while calling Glib::spawn_command_line_sync(): " << ex.what() << std::endl;
-      // TODO: We should cancel the whole call if this fails three times in 
+      std::cerr << "Glom::execute_command_line_and_wait_until_second_command_returns_success(): Exception while calling Glib::spawn_command_line_sync(): " << ex.what() << std::endl;
+      // TODO: We should cancel the whole call if this fails three times in
       // a row or so.
     }
 #else
-    Impl::spawn_sync(second_commands, &stdout_output, NULL, &return_status);
+    return_status = Impl::spawn_sync(second_command, &stdout_output, 0);
 #endif
 
     if(!success_text.empty())
     {
       // Restore the previous environment variable values:
+      #ifdef GLOM_SPAWN_DEBUG
       std::cout << std::endl << "debug: restoring the LANG and LANGUAGE environment variables." << std::endl;
+      #endif //GLOM_SPAWN_DEBUG
+
       Glib::setenv("LANG", stored_env_lang, true /* overwrite */);
       Glib::setenv("LANGUAGE", stored_env_language, true /* overwrite */);
     }
 
-    if(return_status == 0)
+    if(return_status == EXIT_SUCCESS)
     {
       bool success = true; //Just check the return code.
       if(!success_text.empty()) //Check the output too.
       {
+        #ifdef GLOM_SPAWN_DEBUG
         std::cout << " debug: output=" << stdout_output << ", waiting for=" << success_text << std::endl;
+        #endif //GLOM_SPAWN_DEBUG
+
         if(stdout_output.find(success_text) == std::string::npos)
           success = false;
       }
 
       if(success)
       {
-        std::cout << "Success, do response" << std::endl;
+        #ifdef GLOM_SPAWN_DEBUG
+        std::cout << "debug: Success, do response" << std::endl;
+        #endif //GLOM_SPAWN_DEBUG
+
         // Exit from run() in execute_command_line_and_wait_until_second_command_returns_success().
-        dialog_progress->response(Gtk::RESPONSE_OK);
+        mainloop->quit();
         // Cancel timeout. Actually, we also could return true here since
         // the signal is disconnect explicetely after run() anyway.
         return false;
@@ -479,62 +571,94 @@ namespace {
     }
     else
     {
+       #ifdef GLOM_SPAWN_DEBUG
        std::cout << " debug: second command failed. output=" << stdout_output << std::endl;
+       #endif //GLOM_SPAWN_DEBUG
     }
 
-    dialog_progress->pulse();
+    slot_progress(); //Show UI progress feedback.
     return true;
   }
-}
 
-bool execute_command_line_and_wait_until_second_command_returns_success(const std::string& command, const std::string& second_command, const Glib::ustring& message, Gtk::Window* parent_window, const std::string& success_text)
+} //Anonymous namespace
+
+/*
+static bool on_timeout_delay(const Glib::RefPtr<Glib::MainLoop>& mainloop)
 {
-  if(!parent_window)
-    std::cerr << "debug: Glom: execute_command_line_and_wait_until_second_command_returns_success(): parent_window is NULL" << std::endl;
+  //Allow our mainloop.run() to return:
+  if(mainloop)
+    mainloop->quit();
 
-  Dialog_ProgressCreating* dialog_temp = get_and_show_pulse_dialog(message, parent_window);
-  std::auto_ptr<Dialog_ProgressCreating> dialog_progress;
-  dialog_progress.reset(dialog_temp);
+  return false;
+}
+*/
 
-  std::cout << "Command: " << command << std::endl;
+bool execute_command_line_and_wait_until_second_command_returns_success(const std::string& command, const std::string& second_command, const SlotProgress& slot_progress, const std::string& success_text)
+{
+  #ifdef GLOM_SPAWN_DEBUG
+  std::cout << "debug: Command: " << command << std::endl;
+  #endif //GLOM_SPAWN_DEBUG
 
-  std::auto_ptr<const Impl::SpawnInfo> info = Impl::spawn_async(command, Impl::REDIRECT_STDERR);
+  std::auto_ptr<const Impl::SpawnInfo> info;
 
+  try
+  {
+    info = Impl::spawn_async(command, Impl::REDIRECT_STDERR);
+  }
+  catch(const Impl::SpawnError& ex)
+  {
+    std::cerr << G_STRFUNC << ": exception: " << ex.what() << std::endl;
+    return false;
+  }
+  
   // While we wait for the second command to finish we
   // a) check whether the first command finished. If it did, and has a
   // negative error code, we assume it failed and return directly.
   // b) Get stderr data, to display an error message in case the command
   // fails:
 
-  // Hide dialog when the first command finished for some reason
-  sigc::connection watch_conn = info->signal_finished().connect(sigc::bind(sigc::mem_fun(*dialog_temp, &Dialog_ProgressCreating::response), Gtk::RESPONSE_REJECT));
+  Glib::RefPtr<Glib::MainLoop> mainloop = Glib::MainLoop::create(false);
+  sigc::connection watch_conn = info->signal_finished().connect(
+    sigc::bind(sigc::ptr_fun(&on_spawn_info_finished), sigc::ref(mainloop) ) );
 
   // Call the second command once every second
-  sigc::connection timeout_conn = Glib::signal_timeout().connect(sigc::bind(sigc::ptr_fun(&on_timeout), sigc::ref(second_command), sigc::ref(success_text), dialog_temp), 1000);
+  sigc::connection timeout_conn = Glib::signal_timeout().connect(sigc::bind(sigc::ptr_fun(&second_command_on_timeout), sigc::ref(second_command), sigc::ref(success_text), slot_progress, sigc::ref(mainloop)), 1000);
+  slot_progress(); //Make sure it is called at least once.
 
-  // Enter the main loop
-  int response = dialog_temp->run();
+  // Block until signal_finished is emitted:
+  mainloop->run();
 
   timeout_conn.disconnect();
   watch_conn.disconnect();
 
   std::string stderr_text;
+  int return_status = 0;
+  const bool success = Impl::spawn_async_end(info, 0, &stderr_text, &return_status);
 
-  Impl::spawn_async_end(info, NULL, &stderr_text, NULL);
-
-  if(response == Gtk::RESPONSE_OK)
+  if(success && (return_status == EXIT_SUCCESS))
   {
+    /* Don't sleep here. Instead we just keep trying to connect until it succeeds,
+     * timing out during that if necessary.
+     *
+     *
     //Sleep for a bit more, because I think that pg_ctl sometimes reports success too early.
-    Glib::signal_timeout().connect(sigc::bind_return(sigc::ptr_fun(&Gtk::Main::quit), false), 3000);
-    Gtk::Main::run();
+    Glib::RefPtr<Glib::MainLoop> mainloop = Glib::MainLoop::create(false);
+    sigc::connection connection_timeout = Glib::signal_timeout().connect(
+     sigc::bind(sigc::ptr_fun(&on_timeout_delay), sigc::ref(mainloop)),
+     8000);
+    mainloop->run();
+
+    connection_timeout.disconnect();
+    */
 
     return true;
   }
   else
   {
     // The user either cancelled, or the first command failed, or exited prematurely
-    if(response == Gtk::RESPONSE_REJECT)
+    if(true) //response == Gtk::RESPONSE_REJECT)
     {
+      /* TODO: Allow the caller to show a dialog?
       // Command failed
       std::auto_ptr<Gtk::MessageDialog> error_dialog;
       if(parent_window)
@@ -545,6 +669,10 @@ bool execute_command_line_and_wait_until_second_command_returns_success(const st
       // TODO: i18n
       error_dialog->set_secondary_text("The command was:\n\n" + Glib::Markup::escape_text(command) + (stderr_text.empty() ? Glib::ustring("") : ("\n\n<small>" + Glib::Markup::escape_text(stderr_text) + "</small>")), true);
       error_dialog->run();
+      */
+
+      std::cerr << "Glom:  execute_command_line_and_wait_until_second_command_returns_success(): Command failed. The command was: " << command << std::endl <<
+        "and the error was: " << stderr_text << std::endl;
     }
     else
     {
@@ -557,37 +685,6 @@ bool execute_command_line_and_wait_until_second_command_returns_success(const st
   }
 }
 
-#if 0
-bool execute_command_line_and_wait_fixed_seconds(const std::string& command, unsigned int seconds, const Glib::ustring& message, Gtk::Window* parent_window)
-{
-  Gtk::MessageDialog dialog(Bakery::App_Gtk::util_bold_message(message), true, Gtk::MESSAGE_INFO, Gtk::BUTTONS_NONE, true /* modal */); 
-  if(parent_window)
-    dialog.set_transient_for(*parent_window);
-
-  dialog.show();
-
-  //Allow GTK+ to perform all updates for us
-  //Without this, the dialog will seem empty.
-  while(Gtk::Main::instance()->events_pending())
-    Gtk::Main::instance()->iteration();
-
-  std::cout << std::endl << "debug: command_line: " << command << std::endl << std::endl;
-
-  try
-  {
-    Glib::spawn_command_line_async(command);
-  }
-  catch(const Glib::SpawnError& ex)
-  {
-    std::cerr << "Glom::Spawn::execute_command_line_and_wait_fixed_seconds() Exception while calling lib::spawn_command_line_async(): " << ex.what() << std::endl;
-  }
-
-  sleep(seconds); //Give the command enough time to make something ready for us to continue.
-  return true;
-}
-#endif
-
 } //Spawn
 
 } //Glom
-
