@@ -23,9 +23,13 @@
 
 #include <libglom/data_structure/field.h>
 #include <libglom/data_structure/numeric_format.h>
+#include <libglom/document/document.h>
 
 #include <libglom/data_structure/layout/layoutitem_field.h>
 
+#include <libgdamm/sqlexpr.h>
+#include <giomm/file.h>
+
 namespace Glom
 {
 
@@ -46,54 +50,82 @@ typedef std::vector< sharedptr<const LayoutItem_Field> > type_vecConstLayoutFiel
 
 //TODO: Move these to their own file:
 
+// Create a Gnome::Gda::SqlExpr.
+Gnome::Gda::SqlExpr build_simple_where_expression(const Glib::ustring& table_name, const sharedptr<const Field>& key_field, const Gnome::Gda::Value& key_value);
+
+// Create a where clause that is two other conditions combined together.
+Gnome::Gda::SqlExpr build_combined_where_expression(const Gnome::Gda::SqlExpr& a, const Gnome::Gda::SqlExpr& b, Gnome::Gda::SqlOperatorType op);
+
 /** Generate a SQL statement to SELECT field values,
  * even if the fields are in related (or doubly related) records.
  */
-Glib::ustring build_sql_select_fields_to_get(
+void build_sql_select_add_fields_to_get(
+  const Glib::RefPtr<Gnome::Gda::SqlBuilder>& builder,
   const Glib::ustring& table_name,
   const type_vecConstLayoutFields& fieldsToGet,
   const type_sort_clause& sort_clause,
-  Glib::ustring& sql_part_from,
-  Glib::ustring& sql_part_leftouterjoin);
+  bool extra_join);
 
 /** Generate a SQL statement to SELECT field values,
  * even if the fields are in related (or doubly related) records,
  * narrowing the records down with a WHERE clause.
  */
-Glib::ustring build_sql_select_with_where_clause(
+Glib::RefPtr<Gnome::Gda::SqlBuilder> build_sql_select_with_where_clause(
   const Glib::ustring& table_name,
   const type_vecLayoutFields& fieldsToGet,
-  const Glib::ustring& where_clause = Glib::ustring(),
-  const Glib::ustring& extra_join = Glib::ustring(),
+  const Gnome::Gda::SqlExpr& where_clause = Gnome::Gda::SqlExpr(),
+  const sharedptr<const Relationship>& extra_join = sharedptr<const Relationship>(),
   const type_sort_clause& sort_clause = type_sort_clause(),
-  const Glib::ustring& extra_group_by = Glib::ustring());
+  guint limit = 0);
 
 /** Just a version of build_sql_select_with_where_clause() that takes a list of const fields.
  */
-Glib::ustring build_sql_select_with_where_clause(
+Glib::RefPtr<Gnome::Gda::SqlBuilder> build_sql_select_with_where_clause(
   const Glib::ustring& table_name,
   const type_vecConstLayoutFields& fieldsToGet,
-  const Glib::ustring& where_clause = Glib::ustring(),
-  const Glib::ustring& extra_join = Glib::ustring(),
+  const Gnome::Gda::SqlExpr& where_clause = Gnome::Gda::SqlExpr(),
+  const sharedptr<const Relationship>& extra_join = sharedptr<const Relationship>(),
   const type_sort_clause& sort_clause = type_sort_clause(),
-  const Glib::ustring& extra_group_by = Glib::ustring());
+  guint limit = 0);
 
-Glib::ustring build_sql_select_with_key(
+/**
+ * @param key_value If this is empty then all records in the tables will be retrieved.
+ */
+Glib::RefPtr<Gnome::Gda::SqlBuilder> build_sql_select_with_key(
   const Glib::ustring& table_name,
   const type_vecLayoutFields& fieldsToGet,
   const sharedptr<const Field>& key_field,
-  const Gnome::Gda::Value& key_value);
+  const Gnome::Gda::Value& key_value,
+  const type_sort_clause& sort_clause = type_sort_clause(),
+  guint limit = 0);
 
 /** Just a version of build_sql_select_with_key() that takes a list of const fields.
  */
-Glib::ustring build_sql_select_with_key(
+Glib::RefPtr<Gnome::Gda::SqlBuilder> build_sql_select_with_key(
   const Glib::ustring& table_name,
   const type_vecConstLayoutFields& fieldsToGet,
   const sharedptr<const Field>& key_field,
-  const Gnome::Gda::Value& key_value);
+  const Gnome::Gda::Value& key_value,
+  const type_sort_clause& sort_clause = type_sort_clause(),
+  guint limit = 0);
+
+Gnome::Gda::SqlExpr get_find_where_clause_quick(const Document* document, const Glib::ustring& table_name, const Gnome::Gda::Value& quick_search);
+
 
 typedef std::list< std::pair<Gnome::Gda::Value, Gnome::Gda::Value> > type_list_values_with_second;
-type_list_values_with_second get_choice_values(const sharedptr<const LayoutItem_Field>& field);
+type_list_values_with_second get_choice_values_all(const Document* document, const sharedptr<const LayoutItem_Field>& field, sharedptr<const LayoutItem_Field>& layout_choice_first, sharedptr<const LayoutItem_Field>& layout_choice_second);
+
+type_list_values_with_second get_choice_values(const Document* document, const sharedptr<const LayoutItem_Field>& field, const Gnome::Gda::Value& foreign_key_value, sharedptr<const LayoutItem_Field>& layout_choice_first, sharedptr<const LayoutItem_Field>& layout_choice_second);
+
+/// Get the full query string suitable for use with std::cout.
+std::string sqlbuilder_get_full_query(
+  const Glib::RefPtr<Gnome::Gda::Connection>& connection,
+  const Glib::ustring& query,
+  const Glib::RefPtr<const Gnome::Gda::Set>& params);
+
+/// Get the full query string suitable for use with std::cout.
+std::string sqlbuilder_get_full_query(
+  const Glib::RefPtr<const Gnome::Gda::SqlBuilder>& builder);
 
 /** Guess an appropriate identifier name based on a human-readable title
  */
@@ -131,6 +163,20 @@ Glib::ustring string_remove_suffix(const Glib::ustring& str, const Glib::ustring
 
 bool file_exists(const Glib::ustring& uri);
 
+/** Delete a directory, if it exists, and its contents.
+ * Unlike g_file_delete(), this does not fail if the directory is not empty.
+ */
+bool delete_directory(const Glib::RefPtr<Gio::File>& directory);
+
+/** Delete a directory, if it exists, and its contents.
+ * Unlike g_file_delete(), this does not fail if the directory is not empty.
+ */
+bool delete_directory(const std::string& uri);
+
+/** For instance, to find the first file in the directory with a .glom extension.
+ */
+Glib::ustring get_directory_child_with_suffix(const Glib::ustring& uri_directory, const std::string& suffix, bool recursive);
+
 } //namespace Utils
 
 } //namespace Glom
