@@ -19,12 +19,13 @@
  */
 
 #include "box_data_list.h"
-#include <glom/libglom/data_structure/glomconversions.h>
-#include <glom/libglom/glade_utils.h>
-#include <glom/reports/report_builder.h>
-#include "dialog_layout_list.h"
-#include <glom/glom_privs.h>
-#include <bakery/App/App_Gtk.h> //For util_bold_message().
+#include <libglom/data_structure/glomconversions.h>
+#include <glom/glade_utils.h>
+#include <glom/report_builder.h>
+#include <glom/mode_design/layout/dialog_layout_list.h>
+#include <libglom/privs.h>
+#include <libglom/db_utils.h>
+#include <libglom/utils.h> //For bold_message()).
 //#include <../utility_widgets/db_adddel/glom_db_treemodel.h> //For DbTreeModel.
 #include <sstream> //For stringstream
 #include <glibmm/i18n.h>
@@ -42,14 +43,13 @@ Box_Data_List::Box_Data_List()
 
   pack_start(m_AddDel);
   add_view(&m_AddDel); //Give it access to the document.
-  m_AddDel.set_rules_hint(); //Use alternating row colors when the theme does that.
 
   //Connect signals:
   //The Add and Delete buttons are handled by the DbAddDel widget itself.
   m_AddDel.signal_user_requested_edit().connect(sigc::mem_fun(*this, &Box_Data_List::on_adddel_user_requested_edit));
   m_AddDel.signal_script_button_clicked().connect(sigc::mem_fun(*this, &Box_Data_List::on_adddel_script_button_clicked));
   m_AddDel.signal_sort_clause_changed().connect(sigc::mem_fun(*this, &Box_Data_List::on_adddel_user_sort_clause_changed));
-  
+
   //TODO: Re-add this signal if this is really wanted, but make it part of a complete drag-and-drop feature for list views:
   //m_AddDel.signal_user_reordered_columns().connect(sigc::mem_fun(*this, &Box_Data_List::on_adddel_user_reordered_columns));
 
@@ -91,7 +91,7 @@ void Box_Data_List::enable_buttons()
 void Box_Data_List::refresh_data_from_database_blank()
 {
   FoundSet found_set = m_found_set;
-  found_set.m_where_clause = Glib::ustring();
+  found_set.m_where_clause = Gnome::Gda::SqlExpr();
   m_AddDel.set_found_set(found_set);
 
   std::cout << "debug: Box_Data_List::refresh_data_from_database_blank(): before refresh_from_database_blank()." << std::endl;
@@ -108,7 +108,7 @@ bool Box_Data_List::fill_from_database()
   if(!get_document())
     return false;
 
-  Bakery::BusyCursor busy_cursor(get_app_window());
+  BusyCursor busy_cursor(get_app_window());
 
   sharedptr<SharedConnection> sharedconnection;
 
@@ -194,25 +194,25 @@ void Box_Data_List::set_primary_key_value(const Gtk::TreeModel::iterator& row, c
 
 void Box_Data_List::on_adddel_user_reordered_columns()
 {
-  Document_Glom* pDoc = dynamic_cast<Document_Glom*>(get_document());
+  Document* pDoc = dynamic_cast<Document*>(get_document());
   if(pDoc)
   {
     sharedptr<LayoutGroup> group = sharedptr<LayoutGroup>::create();
     group->set_name("toplevel");
 
-    AddDel::type_vecStrings vec_field_names = m_AddDel.get_columns_order();
+    AddDel::type_vec_strings vec_field_names = m_AddDel.get_columns_order();
 
-    for(AddDel::type_vecStrings::iterator iter = vec_field_names.begin(); iter != vec_field_names.end(); ++iter)
+    for(AddDel::type_vec_strings::iterator iter = vec_field_names.begin(); iter != vec_field_names.end(); ++iter)
     {
       sharedptr<LayoutItem_Field> layout_item = sharedptr<LayoutItem_Field>::create();
       layout_item->set_name(*iter);
-      group->add_item(layout_item); 
+      group->add_item(layout_item);
     }
 
-    Document_Glom::type_list_layout_groups mapGroups;
+    Document::type_list_layout_groups mapGroups;
     mapGroups[1] = group;
 
-    pDoc->set_data_layout_groups("list", m_table_name, mapGroups);  
+    pDoc->set_data_layout_groups("list", m_table_name, m_layout_platform, mapGroups);
   }
 }
 
@@ -220,9 +220,25 @@ void Box_Data_List::on_adddel_script_button_clicked(const sharedptr<const Layout
 {
   if(!layout_item)
     return;
-  
+
   const Gnome::Gda::Value primary_key_value = get_primary_key_value(row);
-  execute_button_script(layout_item, primary_key_value);
+
+  // TODO: Calling refresh_data_from_database(),
+  // or navigating to a different table from inside the Python script,
+  // causes a crash somewhere down in GTK+, so it is done in an idle handler here.
+  // We are currently in a callback from the CellRendererButton_Text cell
+  // renderer which is deleted by a call to refresh_data_from_database().
+  // Probably this causes issues somewhere.
+  Glib::signal_idle().connect(
+    sigc::bind(
+      sigc::mem_fun(*this, &Box_Data_List::on_script_button_idle),
+      layout_item,
+      primary_key_value));
+}
+
+bool Box_Data_List::on_script_button_idle(const sharedptr<const LayoutItem_Button>& layout_item, const Gnome::Gda::Value& primary_key)
+{
+  execute_button_script(layout_item, primary_key);
 
   // Refill view from database as the script might have changed arbitrary records
 
@@ -235,21 +251,9 @@ void Box_Data_List::on_adddel_script_button_clicked(const sharedptr<const Layout
     db_model->refresh_from_database(m_found_set);
 #endif
 
-  // TODO: Calling refresh_data_from_database() causes a crash somewhere
-  // down in GTK+, so it is done in a handler here.
-  // We are currently in a callback from the CellRendererButton_Text cell
-  // renderer which is deleted by a call to refresh_data_from_database().
-  // Probably this causes issues somewhere. 
-  Glib::signal_idle().connect(sigc::bind(sigc::mem_fun(*this, &Box_Data_List::on_script_button_idle), primary_key_value));
-
-  //refresh_data_from_database();
-  //set_primary_key_value_selected(primary_key);
-}
-
-bool Box_Data_List::on_script_button_idle(const Gnome::Gda::Value& primary_key)
-{
   refresh_data_from_database();
   set_primary_key_value_selected(primary_key);
+
   return false;
 }
 
@@ -304,7 +308,7 @@ void Box_Data_List::on_details_nav_last()
     m_AddDel.select_item(iter);
     signal_user_requested_details().emit(m_AddDel.get_value_key_selected());
   }
-  
+
   //No, don't do this. When would that ever be a good idea? murrayc:
   //signal_user_requested_details().emit(Gnome::Gda::Value()); //Show a blank record if there are no records.
 }
@@ -429,7 +433,7 @@ void Box_Data_List::create_layout_add_group(const sharedptr<LayoutGroup>& layout
       {
         //Check that the field really exists, to avoid SQL errors.
         //This could probably only happen if we have failed to rename something everywhere, when the user has renamed something.
-        if(!get_field_exists_in_database(child_field->get_table_used(m_table_name), child_field->get_name()))
+        if(!DbUtils::get_field_exists_in_database(child_field->get_table_used(m_table_name), child_field->get_name()))
         {
           std::cerr << "debug: Box_Data_List::create_layout_add_group(): Field does not exist in database: table_name=" << child_field->get_table_used(m_table_name) << ", field_name=" << child_field->get_name() << std::endl;
           continue;
@@ -441,18 +445,18 @@ void Box_Data_List::create_layout_add_group(const sharedptr<LayoutGroup>& layout
   }
 }
 
-Document_Glom::type_list_layout_groups Box_Data_List::create_layout_get_layout()
+Document::type_list_layout_groups Box_Data_List::create_layout_get_layout()
 {
   //This method is overriden in Box_Data_List_Related.
 
-  return get_data_layout_groups(m_layout_name); 
+  return get_data_layout_groups(m_layout_name, m_layout_platform);
 }
 
 void Box_Data_List::create_layout()
 {
   Box_Data::create_layout(); //Fills m_TableFields.
 
-  const Document_Glom* pDoc = dynamic_cast<const Document_Glom*>(get_document());
+  const Document* pDoc = dynamic_cast<const Document*>(get_document());
   if(pDoc)
   {
     //Field Names:
@@ -465,17 +469,19 @@ void Box_Data_List::create_layout()
     sharedptr<Field> field_primary_key = get_field_primary_key_for_table(m_table_name);
     if(!field_primary_key)
     {
-      //g_warning("%s: primary key not found.", __FUNCTION__);
+      std::cerr << "Box_Data_List::create_layout(): primary key not found." << std::endl;
     }
     else
     {
+      //std::cout << "DEBUG: Box_Data_List::create_layout(): primary_key=" << field_primary_key->get_name() << std::endl;
+
       m_AddDel.set_key_field(field_primary_key);
- 
+
       //This map of layout groups will also contain the field information from the database:
-      Document_Glom::type_list_layout_groups layout_groups = create_layout_get_layout();
+      Document::type_list_layout_groups layout_groups = create_layout_get_layout();
 
       //int debug_count = 0;
-      for(Document_Glom::type_list_layout_groups::const_iterator iter = layout_groups.begin(); iter != layout_groups.end(); ++iter)
+      for(Document::type_list_layout_groups::const_iterator iter = layout_groups.begin(); iter != layout_groups.end(); ++iter)
       {
         //std::cout << "Box_Data_List::create_layout() group number=" << debug_count;
         //debug_count++;
@@ -514,7 +520,7 @@ sharedptr<Field> Box_Data_List::get_field_primary_key() const
   return m_AddDel.get_key_field();
 }
 
-void Box_Data_List::print_layout()
+void Box_Data_List::print_layout() //TODO: Is this a duplicate of the one in box_data_manyrecords.cc?
 {
   const Privileges table_privs = Privs::get_current_privs(m_table_name);
 
@@ -576,20 +582,20 @@ void Box_Data_List::get_record_counts(gulong& total, gulong& found) const
 
   Glib::RefPtr<Gtk::TreeModel> refModel = m_AddDel.get_model();
   Glib::RefPtr<DbTreeModel> refModelDerived = Glib::RefPtr<DbTreeModel>::cast_dynamic(refModel);
-  
+
   if(refModelDerived)
     refModelDerived->get_record_counts(total, found);
 }
 
 void Box_Data_List::on_adddel_user_sort_clause_changed()
 {
-  //Remember details about the previously viewed table, 
-  //so we don't forget the sort order and where clause when 
+  //Remember details about the previously viewed table,
+  //so we don't forget the sort order and where clause when
   //navigating back, which would annoy the user:
 
   m_found_set = m_AddDel.get_found_set();
 
-  Document_Glom* document = get_document(); 
+  Document* document = get_document();
   if(document)
     document->set_criteria_current(m_table_name, m_found_set);
 }
@@ -608,22 +614,15 @@ void Box_Data_List::on_dialog_layout_hide()
 
 Dialog_Layout* Box_Data_List::create_layout_dialog() const
 {
-  Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(Utils::get_glade_file_path("glom_developer.glade"), "window_data_layout"); //TODO: Use a generic layout dialog?
-  if(refXml)
-  {
-    Dialog_Layout_List* dialog = 0;
-    refXml->get_widget_derived("window_data_layout", dialog);
-    return dialog;
-  }
-
-  return NULL;
+  Dialog_Layout_List* dialog = 0;
+  Glom::Utils::get_glade_widget_derived_with_warning(dialog);
+  return dialog;
 }
 
 void Box_Data_List::prepare_layout_dialog(Dialog_Layout* dialog)
 {
-  dialog->set_document(m_layout_name, get_document(), m_table_name, m_FieldsShown); //TODO: Use m_TableFields?
+  dialog->set_document(m_layout_name, m_layout_platform, get_document(), m_table_name, m_FieldsShown); //TODO: Use m_TableFields?
 }
 #endif // !GLOM_ENABLE_CLIENT_ONLY
 
 } //namespace Glom
-
