@@ -28,21 +28,22 @@
 #else
 #include <glom/utility_widgets/flowtable_dnd.h>
 #endif
-#include <glom/libglom/data_structure/layout/layoutgroup.h>
-#include <glom/libglom/data_structure/layout/layoutitem_field.h>
-#include <glom/libglom/data_structure/layout/layoutitem_notebook.h>
-#include <glom/libglom/data_structure/layout/layoutitem_portal.h>
-#include <glom/libglom/data_structure/layout/layoutitem_calendarportal.h>
-#include <glom/libglom/data_structure/layout/layoutitem_button.h>
-#include <glom/libglom/data_structure/layout/layoutitem_text.h>
-#include <glom/libglom/data_structure/layout/layoutitem_placeholder.h>
-#include <glom/libglom/data_structure/field.h>
-#include <glom/libglom/document/document_glom.h>
+#include <libglom/data_structure/layout/layoutgroup.h>
+#include <libglom/data_structure/layout/layoutitem_field.h>
+#include <libglom/data_structure/layout/layoutitem_notebook.h>
+#include <libglom/data_structure/layout/layoutitem_portal.h>
+#include <libglom/data_structure/layout/layoutitem_calendarportal.h>
+#include <libglom/data_structure/layout/layoutitem_button.h>
+#include <libglom/data_structure/layout/layoutitem_text.h>
+#include <libglom/data_structure/layout/layoutitem_placeholder.h>
+#include <libglom/data_structure/field.h>
+#include <libglom/document/document.h>
 #include <glom/utility_widgets/layoutwidgetbase.h>
 #include <glom/utility_widgets/layoutwidgetutils.h>
-#include "box_data_list_related.h"
+#include <glom/mode_data/box_data_list_related.h>
+#include <glom/mode_data/datawidget/combochoices.h>
 #include "box_data_calendar_related.h"
-#include "treestore_layout.h" //Forthe enum.
+#include <glom/mode_design/layout/treestore_layout.h> //Forthe enum.
 #include <map>
 #include <list>
 
@@ -52,7 +53,7 @@ namespace Glom
 class DataWidget;
 
 class FlowTableWithFields
-  : 
+  :
 #ifdef GLOM_ENABLE_CLIENT_ONLY
     public FlowTable,
     public LayoutWidgetUtils,
@@ -61,7 +62,7 @@ class FlowTableWithFields
 #endif
     public View_Composite_Glom
 {
-public: 
+public:
   FlowTableWithFields(const Glib::ustring& table_name = Glib::ustring());
   virtual ~FlowTableWithFields();
 
@@ -72,22 +73,27 @@ public:
    * @param layoutitem_field The layout item that describes this field,
    * @param table_name The table on which this layout appears.
    */
-  virtual void add_field(const sharedptr<LayoutItem_Field>& layoutitem_field, const Glib::ustring& table_name);
-  virtual void remove_field(const Glib::ustring& id);
+  void add_field(const sharedptr<LayoutItem_Field>& layoutitem_field, const Glib::ustring& table_name);
+
+  void remove_field(const Glib::ustring& id);
 
   typedef std::map<int, Field> type_map_field_sequence;
   //virtual void add_group(const Glib::ustring& group_name, const Glib::ustring& group_title, const type_map_field_sequence& fields);
 
-  virtual void add_layout_item(const sharedptr<LayoutItem>& item);
-  virtual void add_layout_group(const sharedptr<LayoutGroup>& group);
-	
-  virtual void set_field_editable(const sharedptr<const LayoutItem_Field>& field, bool editable = true);
+  void add_layout_item(const sharedptr<LayoutItem>& item);
+  void add_layout_group(const sharedptr<LayoutGroup>& group);
 
-  virtual Gnome::Gda::Value get_field_value(const sharedptr<const LayoutItem_Field>& field) const;
-  //virtual Gnome::Gda::Value get_field_value(const Glib::ustring& id) const;
-  virtual void set_field_value(const sharedptr<const LayoutItem_Field>& field, const Gnome::Gda::Value& value);
-  //virtual void set_field_value(const Glib::ustring& id, const Gnome::Gda::Value& value);
+  void set_field_editable(const sharedptr<const LayoutItem_Field>& field, bool editable = true);
 
+  Gnome::Gda::Value get_field_value(const sharedptr<const LayoutItem_Field>& field) const;
+
+  /** Set the displayed @a value in any instances of the specified @a field.
+   */
+  void set_field_value(const sharedptr<const LayoutItem_Field>& field, const Gnome::Gda::Value& value);
+
+  /** Set the displayed @a value in any instances of the field other than the specified @a layout_field.
+   */
+  void set_other_field_value(const sharedptr<const LayoutItem_Field>& layout_field, const Gnome::Gda::Value& value);
 
   typedef std::list<Gtk::Widget*> type_list_widgets;
   typedef std::list<const Gtk::Widget*> type_list_const_widgets;
@@ -98,12 +104,26 @@ public:
 
   virtual void remove_all();
 
-  /** Get the layout structure, which might have changed in the child widgets since 
+  typedef std::vector< Glib::RefPtr<Gtk::SizeGroup> > type_vec_sizegroups;
+
+  /** Apply the size groups to all field labels.
+   * By calling this method on multiple FlowTables, the field widgets in
+   * different groups can then align.
+   * @param size_groups A vector containing a size group for each possible column.
+   */
+  void apply_size_groups_to_labels(const type_vec_sizegroups& size_group);
+
+  /** Create a size group and make all the labels in child flowtables use it,
+   * making them align.
+   */
+  void align_child_group_labels();
+
+  /** Get the layout structure, which might have changed in the child widgets since
    * the whole widget structure was built.
-   * for instance, if the user chose a new field for a DataWidget, 
+   * for instance, if the user chose a new field for a DataWidget,
    * or a new relationship for a portal.
    */
-  void get_layout_groups(Document_Glom::type_list_layout_groups& groups);
+  void get_layout_groups(Document::type_list_layout_groups& groups);
   sharedptr<LayoutGroup> get_layout_group();
 
   /** For instance,
@@ -111,7 +131,7 @@ public:
    */
   typedef sigc::signal<void, const sharedptr<const LayoutItem_Field>&, const Gnome::Gda::Value&> type_signal_field_edited;
   type_signal_field_edited signal_field_edited();
-  
+
   /** For instance,
    * void on_flowtable_field_open_details_requested(const sharedptr<const LayoutItem_Field>& field, const Gnome::Gda::Value& value);
    */
@@ -135,23 +155,38 @@ public:
    */
   typedef sigc::signal<void, const sharedptr<LayoutItem_Button>&> type_signal_script_button_clicked;
   type_signal_script_button_clicked signal_script_button_clicked();
-  
-protected:
 
-  type_list_widgets get_field(const sharedptr<const LayoutItem_Field>& field);
-  type_list_const_widgets get_field(const sharedptr<const LayoutItem_Field>& field) const;
+private:
+
+  void set_field_value(const sharedptr<const LayoutItem_Field>& field, const Gnome::Gda::Value& value, bool set_specified_field_layout);
+
+  // If include_item is set, then the output list will contain field's widget,
+  // otherwise not.
+  type_list_widgets get_field(const sharedptr<const LayoutItem_Field>& field, bool include_item);
+  type_list_const_widgets get_field(const sharedptr<const LayoutItem_Field>& field, bool include_item) const;
+
+  typedef std::list<Box_Data_Portal*> type_portals;
 
-  typedef std::list< Box_Data_Portal* > type_portals;
-    
   /// Get portals whose relationships have @a from_key as the from_key.
   type_portals get_portals(const sharedptr<const LayoutItem_Field>& from_key);
 
+
+  typedef std::list<DataWidgetChildren::ComboChoices*> type_choice_widgets;
+
+  /// Get choice widgets with !show_all relationships that have @a from_key as the from_key.
+  type_choice_widgets get_choice_widgets(const sharedptr<const LayoutItem_Field>& from_key);
+
+  /** Examine this flow table and all child flow tables, discovering which
+   * has the most columns.
+   */
+  guint get_sub_flowtables_max_columns() const;
+
   //int get_suitable_width(Field::glom_field_type field_type);
 
-  void on_entry_edited(const Gnome::Gda::Value& value, sharedptr<const LayoutItem_Field> field);
-  void on_entry_open_details_requested(const Gnome::Gda::Value& value, sharedptr<const LayoutItem_Field> field);
+  void on_entry_edited(const Gnome::Gda::Value& value, const sharedptr<const LayoutItem_Field> field);
+  void on_entry_open_details_requested(const Gnome::Gda::Value& value, const sharedptr<const LayoutItem_Field> field);
   void on_flowtable_entry_edited(const sharedptr<const LayoutItem_Field>& field, const Gnome::Gda::Value& value);
-  void on_flowtable_entry_open_details_requested(const sharedptr<const LayoutItem_Field>& field, const Gnome::Gda::Value& value); 
+  void on_flowtable_entry_open_details_requested(const sharedptr<const LayoutItem_Field>& field, const Gnome::Gda::Value& value);
   void on_flowtable_related_record_changed(const Glib::ustring& relationship_name);
   void on_flowtable_requested_related_details(const Glib::ustring& table_name, Gnome::Gda::Value primary_key_value);
 
@@ -173,9 +208,11 @@ protected:
     Info();
 
     sharedptr<const LayoutItem_Field> m_field; //Store the field information so we know the title, ID, and type.
-    Glib::ustring m_group;
 
     Gtk::Label* m_first;
+    Gtk::EventBox* m_first_eventbox; //The label is often inside an eventbox.
+    Glib::RefPtr<Gtk::SizeGroup> m_first_in_sizegroup; //Just to avoid a warning when removing a widget not in a group.
+
     DataWidget* m_second;
     Gtk::CheckButton* m_checkbutton; //Used instead of first and second if it's a bool.
   };
@@ -206,37 +243,44 @@ protected:
   void add_layout_notebook_at_position(const sharedptr<LayoutItem_Notebook>& notebook, const type_list_layoutwidgets::iterator& add_before);
   void add_layout_portal_at_position(const sharedptr<LayoutItem_Portal>& portal, const type_list_layoutwidgets::iterator& add_before);
 
+  virtual void on_size_allocate(Gtk::Allocation& allocation);
+
 #ifndef GLOM_ENABLE_CLIENT_ONLY
-  virtual void on_dnd_add_placeholder(LayoutWidgetBase* above);
+
+  virtual void on_dnd_add_layout_item_by_type(int item_type_num, Gtk::Widget* above);
+  virtual void on_dnd_add_placeholder(Gtk::Widget* above);
   virtual void on_dnd_remove_placeholder();
-    
-  // Methods for the different layout object
-  virtual void on_dnd_add_layout_item_field (LayoutWidgetBase* above);
-  virtual void on_dnd_add_layout_group(LayoutWidgetBase* above);
-  virtual void on_dnd_add_layout_item_button (LayoutWidgetBase* above);
-  virtual void on_dnd_add_layout_item_text (LayoutWidgetBase* above);
-  virtual void on_dnd_add_layout_item_image (LayoutWidgetBase* above);
-  virtual void on_dnd_add_layout_notebook (LayoutWidgetBase* above);
-  virtual void on_dnd_add_layout_item (LayoutWidgetBase* above,
-                                       sharedptr<LayoutItem>& item);
-  
-  bool get_field_information (sharedptr<LayoutItem>& item);
+  virtual void set_child_widget_dnd_in_progress(Gtk::Widget* child, bool in_progress);
+  virtual bool get_child_widget_dnd_in_progress(Gtk::Widget* child) const;
+
+
+  void on_dnd_add_layout_item_field(LayoutWidgetBase* above);
+  void on_dnd_add_layout_group(LayoutWidgetBase* above);
+  void on_dnd_add_layout_item_button(LayoutWidgetBase* above);
+  void on_dnd_add_layout_item_text(LayoutWidgetBase* above);
+  void on_dnd_add_layout_item_image(LayoutWidgetBase* above);
+  void on_dnd_add_layout_notebook(LayoutWidgetBase* above);
+  void on_dnd_add_layout_portal(LayoutWidgetBase* above);
+  void on_dnd_add_layout_item(LayoutWidgetBase* above, const sharedptr<LayoutItem>& item);
+
+  sharedptr<LayoutItem_Portal> get_portal_relationship();
+
   void dnd_notify_failed_drop();
-  bool dnd_add_to_layout_group (sharedptr<LayoutItem>& item, 
-                                LayoutWidgetBase* layoutwidget,
-                                bool ignore_error = false);
-  
+  bool dnd_add_to_layout_group(const sharedptr<LayoutItem>& item, LayoutWidgetBase* layoutwidget, bool ignore_error = false);
+
 #endif // !GLOM_ENABLE_CLIENT_ONLY
 
-  sharedptr<LayoutItem_Portal> get_layout_item_from_relation();
-  
   Box_Data_List_Related* create_related(const sharedptr<LayoutItem_Portal>& portal, bool show_title = true);
   Box_Data_Calendar_Related* create_related_calendar(const sharedptr<LayoutItem_CalendarPortal>& portal, bool show_title = true);
 
   Gtk::Alignment* m_placeholder;
-  
+
   Glib::ustring m_table_name;
 
+  //Size groups shared by this widget's sibling FlowTables,
+  //with one group for each column.
+  type_vec_sizegroups m_vec_size_groups;
+
   type_signal_field_edited m_signal_field_edited;
   type_signal_field_open_details_requested m_signal_field_open_details_requested;
 
@@ -244,7 +288,7 @@ protected:
   type_signal_related_record_changed m_signal_related_record_changed;
   type_signal_requested_related_details m_signal_requested_related_details;
   type_signal_script_button_clicked m_signal_script_button_clicked;
-    
+
   //menu
 #ifndef GLOM_ENABLE_CLIENT_ONLY
   virtual void on_menu_properties_activate();
