@@ -20,9 +20,10 @@
 
 
 #include "dialog_script_library.h"
-#include "../../python_embed/glom_python.h"
-#include <glom/libglom/data_structure/glomconversions.h>
-#include <glom/libglom/glade_utils.h>
+#include <glom/python_embed/glom_python.h>
+#include <libglom/data_structure/glomconversions.h>
+#include <glom/glade_utils.h>
+#include <glom/utils_ui.h>
 #include <glom/mode_design/script_library/dialog_new_script.h>
 #include <gtksourceviewmm/sourcelanguagemanager.h>
 #include <glom/application.h>
@@ -34,15 +35,18 @@
 namespace Glom
 {
 
-Dialog_ScriptLibrary::Dialog_ScriptLibrary(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml>& refGlade)
+const char* Dialog_ScriptLibrary::glade_id("dialog_script_library");
+const bool Dialog_ScriptLibrary::glade_developer(true);
+
+Dialog_ScriptLibrary::Dialog_ScriptLibrary(BaseObjectType* cobject, const Glib::RefPtr<Gtk::Builder>& builder)
 : Gtk::Dialog(cobject)
 {
   //Get child widgets:
-  refGlade->get_widget_derived("combobox_name", m_combobox_name);
-  refGlade->get_widget("textview_script",  m_text_view);
-  refGlade->get_widget("button_check",  m_button_check);
-  refGlade->get_widget("button_add",  m_button_add);
-  refGlade->get_widget("button_remove",  m_button_remove);
+  builder->get_widget_derived("combobox_name", m_combobox_name);
+  builder->get_widget("textview_script",  m_text_view);
+  builder->get_widget("button_check",  m_button_check);
+  builder->get_widget("button_add",  m_button_add);
+  builder->get_widget("button_remove",  m_button_remove);
 
   //Connect signals:
   m_button_check->signal_clicked().connect( sigc::mem_fun(*this, &Dialog_ScriptLibrary::on_button_check) );
@@ -84,24 +88,14 @@ void Dialog_ScriptLibrary::on_button_add()
   //Save any outstanding changes:
   save_current_script();
 
-  Document_Glom* document = get_document();
+  Document* document = get_document();
   if(!document)
     return;
 
   Dialog_NewScript* dialog = 0;
-  try
-  {
-    Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(Utils::get_glade_file_path("glom_developer.glade"), "dialog_new_library_script");
-
-    refXml->get_widget_derived("dialog_new_library_script", dialog);
-  }
-  catch(const Gnome::Glade::XmlError& ex)
-  {
-    std::cerr << ex.what() << std::endl;
-  }
-
+  Utils::get_glade_widget_derived_with_warning(dialog);
   dialog->set_transient_for(*this);
-  const int response = Glom::Utils::dialog_run_with_help(dialog, "dialog_new_script");
+  const int response = Glom::Utils::dialog_run_with_help(dialog);
   dialog->hide();
   if(response != Gtk::RESPONSE_OK)
     return;
@@ -125,11 +119,11 @@ void Dialog_ScriptLibrary::on_button_add()
 
 void Dialog_ScriptLibrary::on_button_remove()
 {
-  Document_Glom* document = get_document();
+  Document* document = get_document();
   if(!document)
     return;
 
-  Gtk::MessageDialog dialog(Bakery::App_Gtk::util_bold_message(_("Remove library script")), true, Gtk::MESSAGE_QUESTION, Gtk::BUTTONS_NONE );
+  Gtk::MessageDialog dialog(Utils::bold_message(_("Remove library script")), true, Gtk::MESSAGE_QUESTION, Gtk::BUTTONS_NONE );
   dialog.set_secondary_text(_("Do you really want to delete this script? This data can not be recovered"));
   dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
   dialog.add_button(Gtk::Stock::REMOVE, Gtk::RESPONSE_OK);
@@ -159,7 +153,7 @@ void Dialog_ScriptLibrary::on_combo_name_changed()
 
 void Dialog_ScriptLibrary::load_current_script()
 {
-  Document_Glom* document = get_document();
+  Document* document = get_document();
   if(!document)
     return;
 
@@ -181,7 +175,7 @@ void Dialog_ScriptLibrary::load_current_script()
 
 void Dialog_ScriptLibrary::save_current_script()
 {
-  Document_Glom* document = get_document();
+  Document* document = get_document();
   if(!document)
     return;
 
@@ -200,12 +194,12 @@ void Dialog_ScriptLibrary::save_current_script()
 
 void Dialog_ScriptLibrary::load_from_document()
 {
-  Document_Glom* document = get_document();
+  Document* document = get_document();
   if(!document)
     return;
 
   const std::vector<Glib::ustring> module_names = document->get_library_module_names();
-  m_combobox_name->clear_text();
+  m_combobox_name->clear_items();
  
   for(std::vector<Glib::ustring>::const_iterator iter = module_names.begin(); iter != module_names.end(); ++iter)
   {
@@ -233,7 +227,7 @@ void Dialog_ScriptLibrary::load_from_document()
 
 void Dialog_ScriptLibrary::save_to_document()
 {
-  Document_Glom* document = get_document();
+  Document* document = get_document();
   if(!document)
     return;
 
