@@ -26,6 +26,7 @@
 #include <libglom/data_structure/layout/report_parts/layoutitem_verticalgroup.h>
 #include <libglom/data_structure/layout/report_parts/layoutitem_header.h>
 #include <libglom/data_structure/layout/report_parts/layoutitem_footer.h>
+#include <libglom/db_utils.h>
 #include <glom/xsl_utils.h>
 
 namespace Glom
@@ -176,27 +177,32 @@ void ReportBuilder::report_build_groupby(const FoundSet& found_set_parent, xmlpp
 
     //Get the possible group values, ignoring repeats by using GROUP BY.
     const Glib::ustring group_field_table_name = field_group_by->get_table_used(found_set_parent.m_table_name);
-    Glib::ustring sql_query = "SELECT \"" + group_field_table_name + "\".\"" + field_group_by->get_name() + "\""
-      " FROM \"" + group_field_table_name + "\"";
+
+    Glib::RefPtr<Gnome::Gda::SqlBuilder> builder =
+      Gnome::Gda::SqlBuilder::create(Gnome::Gda::SQL_STATEMENT_SELECT);
+    builder->select_add_field(field_group_by->get_name(), group_field_table_name);
+    builder->select_add_target(group_field_table_name);
 
     if(!found_set_parent.m_where_clause.empty())
-      sql_query += " WHERE " + found_set_parent.m_where_clause;
+    {
+      builder->set_where( builder->import_expression(found_set_parent.m_where_clause) );
+    }
 
-    sql_query += " GROUP BY " + field_group_by->get_name(); //rTODO: And restrict to the current found set.
+    builder->select_group_by( builder->add_field_id(field_group_by->get_name(), group_field_table_name) ); //TODO: And restrict to the current found set.
 
-    Glib::RefPtr<Gnome::Gda::DataModel> datamodel = query_execute_select(sql_query);
+    Glib::RefPtr<Gnome::Gda::DataModel> datamodel = DbUtils::query_execute_select(builder);
     if(datamodel)
     {
       guint rows_count = datamodel->get_n_rows();
       for(guint row = 0; row < rows_count; ++row)
       {
-#ifdef GLIBMM_EXCEPTIONS_ENABLED      
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
         const Gnome::Gda::Value group_value = datamodel->get_value_at(0 /* col*/, row);
 #else
         std::auto_ptr<Glib::Error> error;
         const Gnome::Gda::Value group_value = datamodel->get_value_at(0 /* col*/, row, error);
-#endif        
-        
+#endif
+
 
         //Add XML node:
         xmlpp::Element* nodeGroupBy = parent_node.add_child(group_by->get_report_part_id());
@@ -207,9 +213,14 @@ void ReportBuilder::report_build_groupby(const FoundSet& found_set_parent, xmlpp
           Conversions::get_text_for_gda_value(field_group_by->get_glom_type(), group_value, field_group_by->get_formatting_used().m_numeric_format) );
 
         //TODO: Use a SQL parameter instead of using sql().
-        Glib::ustring where_clause = "(\"" + group_field_table_name + "\".\"" + field_group_by->get_name() + "\" = " + field_group_by->get_full_field_details()->sql(group_value) + ")";
+        Gnome::Gda::SqlExpr where_clause =
+          Utils::build_simple_where_expression(group_field_table_name, field_group_by->get_full_field_details(), group_value);
+
         if(!found_set_parent.m_where_clause.empty())
-          where_clause += " AND (" + found_set_parent.m_where_clause + ')';
+        {
+          where_clause = Utils::build_combined_where_expression(where_clause, found_set_parent.m_where_clause,
+            Gnome::Gda::SQL_OPERATOR_TYPE_AND);
+        }
 
         FoundSet found_set_records = found_set_parent;
         found_set_records.m_where_clause = where_clause;
@@ -277,7 +288,7 @@ void ReportBuilder::report_build_records(const FoundSet& found_set, xmlpp::Eleme
       sharedptr<LayoutItem> layout_item = *iter;
       sharedptr<LayoutItem_Field> layoutitem_field = sharedptr<LayoutItem_Field>::cast_dynamic(layout_item);
 
-      //This adds a field heading (and therefore, column) for fields, or for a vertical group. 
+      //This adds a field heading (and therefore, column) for fields, or for a vertical group.
       xmlpp::Element* nodeFieldHeading = parent_node.add_child("field_heading");
       if(layoutitem_field && layoutitem_field->get_glom_type() == Field::TYPE_NUMERIC)
         nodeFieldHeading->set_attribute("field_type", "numeric"); //TODO: More sophisticated formatting.
@@ -305,16 +316,18 @@ void ReportBuilder::report_build_records(const FoundSet& found_set, xmlpp::Eleme
       }
     }
 
-   Glib::ustring sql_query = Utils::build_sql_select_with_where_clause(found_set.m_table_name,
-      fieldsToGet,
-      found_set.m_where_clause, Glib::ustring() /* extra_join */, found_set.m_sort_clause);
-
     //For instance, when we just want to get a name corresponding to a contact ID, and want to ignore duplicates.
+    guint limit = 0; //no limit.
     if(one_record_only)
-      sql_query += " LIMIT 1";
+      limit = 1;
+
+    Glib::RefPtr<Gnome::Gda::SqlBuilder> sql_query = Utils::build_sql_select_with_where_clause(found_set.m_table_name,
+      fieldsToGet,
+      found_set.m_where_clause, sharedptr<const Relationship>() /* extra_join */, found_set.m_sort_clause,
+      limit);
 
     bool records_found = false;
-    Glib::RefPtr<Gnome::Gda::DataModel> datamodel = query_execute_select(sql_query);
+    Glib::RefPtr<Gnome::Gda::DataModel> datamodel = DbUtils::query_execute_select(sql_query);
     if(datamodel)
     {
       const guint rows_count = datamodel->get_n_rows();
@@ -384,30 +397,32 @@ void ReportBuilder::report_build_records_field(const FoundSet& found_set, xmlpp:
   {
     //In this case it can only be a system preferences field.
     //So let's get that data here:
-    const Glib::ustring table_used = field->get_table_used(found_set.m_table_name);
-    const Glib::ustring query = "SELECT \"" + table_used + "\".\"" + field->get_name() + "\" FROM \""+ table_used + "\" LIMIT 1";
-    Glib::RefPtr<Gnome::Gda::DataModel> datamodel = query_execute_select(query);
+    Glib::RefPtr<Gnome::Gda::SqlBuilder> builder = Gnome::Gda::SqlBuilder::create(Gnome::Gda::SQL_STATEMENT_UPDATE);
+    builder->set_table(field->get_table_used(found_set.m_table_name));
+    builder->select_add_field(field->get_name(), found_set.m_table_name);
+    builder->select_set_limit(1);
+    Glib::RefPtr<Gnome::Gda::DataModel> datamodel = DbUtils::query_execute_select(builder);
 
     if(!datamodel)
       return;
 
-#ifdef GLIBMM_EXCEPTIONS_ENABLED      
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
     value = datamodel->get_value_at(colField, row);
 #else
     std::auto_ptr<Glib::Error> error;
     value = datamodel->get_value_at(colField, row, error);
-#endif 
+#endif
     colField = 0;
     row = 0;
   }
   else
   {
-#ifdef GLIBMM_EXCEPTIONS_ENABLED      
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
     value = datamodel->get_value_at(colField, row);
 #else
     std::auto_ptr<Glib::Error> error;
     value = datamodel->get_value_at(colField, row, error);
-#endif 
+#endif
   }
 
   nodeField->set_attribute("title", field->get_title_or_name()); //Not always used, but useful.
