@@ -81,6 +81,7 @@ DbAddDel::DbAddDel()
   m_allow_delete(true),
   m_find_mode(false),
   m_allow_only_one_related_record(false),
+  m_validation_retry(false),
   m_columns_ready(false),
   m_allow_view(true),
   m_allow_view_details(false),
@@ -295,7 +296,7 @@ void DbAddDel::setup_menu()
   try
   {
 #endif
-    Glib::ustring ui_info = 
+    Glib::ustring ui_info =
         "<ui>"
         "  <popup name='ContextMenu'>"
         "    <menuitem action='ContextEdit'/>"
@@ -813,6 +814,8 @@ Gtk::CellRenderer* DbAddDel::construct_specified_columns_cellrenderer(const shar
     //Connect to edited signal:
     if(item_field) //Only fields can be edited:
     {
+      pCellRendererText->signal_editing_started().connect(sigc::mem_fun(*this, &DbAddDel::on_treeview_cell_editing_started));
+
       //Make it editable:
       pCellRendererText->set_property("editable", true);
 
@@ -908,9 +911,9 @@ void DbAddDel::apply_formatting(Gtk::CellRenderer* renderer, const sharedptr<con
   const FieldFormatting::HorizontalAlignment alignment =
     layout_item->get_formatting_used_horizontal_alignment();
   const float x_align = (alignment == FieldFormatting::HORIZONTAL_ALIGNMENT_LEFT ? 0.0 : 1.0);
-#ifdef GLIBMM_PROPERTIES_ENABLED  
+#ifdef GLIBMM_PROPERTIES_ENABLED
       text_renderer->property_xalign() = x_align;
-#else    
+#else
       text_renderer->set_property("xalign", alignment);
 #endif
 
@@ -918,25 +921,25 @@ void DbAddDel::apply_formatting(Gtk::CellRenderer* renderer, const sharedptr<con
 
   const Glib::ustring font_desc = formatting.get_text_format_font();
   if(!font_desc.empty())
-#ifdef GLIBMM_PROPERTIES_ENABLED  
+#ifdef GLIBMM_PROPERTIES_ENABLED
     text_renderer->property_font() = font_desc;
 #else
     text_renderer->set_property("font", font_desc);
-#endif        
+#endif
 
   const Glib::ustring fg = formatting.get_text_format_color_foreground();
   if(!fg.empty())
-#ifdef GLIBMM_PROPERTIES_ENABLED  
+#ifdef GLIBMM_PROPERTIES_ENABLED
     text_renderer->property_foreground() = fg;
-#else    
+#else
     text_renderer->set_property("foreground", fg);
 #endif
-    
+
   const Glib::ustring bg = formatting.get_text_format_color_background();
   if(!bg.empty())
 #ifdef GLIBMM_PROPERTIES_ENABLED
     text_renderer->property_background() = bg;
-#else    
+#else
     text_renderer->set_property("background", bg);
 #endif
 }
@@ -1809,6 +1812,36 @@ void DbAddDel::on_treeview_cell_edited_bool(const Glib::ustring& path_string, in
   }
 }
 
+void DbAddDel::on_idle_treeview_cell_edited_revert(const Gtk::TreeModel::Row& row, guint model_column_index)
+{
+  Glib::RefPtr<Gtk::TreeSelection> refTreeSelection = m_TreeView.get_selection();
+  if(!refTreeSelection)
+    return;
+
+  refTreeSelection->select(row); //TODO: This does not seem to work.
+
+  guint view_column_index = 0;
+  get_view_column_index(model_column_index, view_column_index);
+  Gtk::TreeView::Column* pColumn = m_TreeView.get_column(view_column_index);
+  if(!pColumn)
+  {
+    std::cerr << G_STRFUNC << ": pColumn is null." << std::endl;
+    return;
+  }
+
+  Gtk::CellRendererText* pCell = dynamic_cast<Gtk::CellRendererText*>(pColumn->get_first_cell_renderer());
+  if(!pCell)
+  {
+    std::cerr << G_STRFUNC << ": pCell is null." << std::endl;
+    return;
+  }
+
+  const Gtk::TreeModel::Path path = get_model()->get_path(row);
+
+  //Highlights the cell, and start the editing:
+  m_TreeView.set_cursor(path, *pColumn, *pCell, true /* start_editing */);
+}
+
 void DbAddDel::on_treeview_cell_edited(const Glib::ustring& path_string, const Glib::ustring& new_text, int model_column_index, int data_model_column_index)
 {
   //Note:: model_column_index is actually the AddDel column index, not the TreeModel column index.
@@ -1820,6 +1853,12 @@ void DbAddDel::on_treeview_cell_edited(const Glib::ustring& path_string, const G
 
   const Gtk::TreeModel::Path path(path_string);
 
+  if(path.empty())
+  {
+    std::cerr << G_STRFUNC << ": path is empty." << std::endl;
+    return;
+  }
+
   //Get the row from the path:
   Gtk::TreeModel::iterator iter = m_refListStore->get_iter(path);
   if(iter != get_model()->children().end())
@@ -1876,7 +1915,7 @@ void DbAddDel::on_treeview_cell_edited(const Glib::ustring& path_string, const G
       if(!success)
       {
           //Tell the user and offer to revert or try again:
-          bool revert = glom_show_dialog_invalid_data(field_type);
+          const bool revert = glom_show_dialog_invalid_data(field_type);
           if(revert)
           {
             //Revert the data:
@@ -1886,43 +1925,16 @@ void DbAddDel::on_treeview_cell_edited(const Glib::ustring& path_string, const G
           {
             //Reactivate the cell so that the data can be corrected.
 
-            Glib::RefPtr<Gtk::TreeSelection> refTreeSelection = m_TreeView.get_selection();
-            if(refTreeSelection)
-            {
-              refTreeSelection->select(row); //TODO: This does not seem to work.
+            //Set the text to be used in the start_editing signal handler:
+            m_validation_invalid_text_for_retry = new_text;
+            m_validation_retry = true;
 
-              if(!path.empty())
-              {
-                Gtk::TreeView::Column* pColumn = m_TreeView.get_column(model_column_index);
-                if(pColumn)
-                {
-                  Gtk::CellRendererText* pCell = dynamic_cast<Gtk::CellRendererText*>(pColumn->get_first_cell_renderer());
-                  if(pCell)
-                  {
-                    //TreeView::set_cursor(), or start_editing() would get the old value back from the model again
-                    //so we do something similar without getting the old value:
-                    m_TreeView.set_cursor(path, *pColumn, *pCell, true /* start_editing */); //This highlights the cell, and starts the editing.
-
-                    //This is based on gtk_tree_view_start_editing():
-                    //TODO: This does not actually work. I emailed gtk-list about how to do this.
-                    /*
-                    pCell->stop_editing();
-                    pCell->property_text() = "test"; //new_text; //Allow the user to start with the bad text that he entered so far.
-
-                    Gdk::Rectangle background_area;
-                    m_TreeView.get_background_area(path, *pColumn, background_area);
-
-                    Gdk::Rectangle cell_area;
-                    m_TreeView.get_cell_area(path, *pColumn, background_area);
-                    */
-                  }
-                }
-              }
-              else
-              {
-                g_warning("DbAddDel::on_treeview_cell_edited(): path is invalid.");
-              }
-            }
+            //But do this in an idle timout, so that the TreeView doesn't get
+            //confused by us changing editing state in this signal handler.
+            Glib::signal_idle().connect_once(
+              sigc::bind(
+               sigc::mem_fun(*this, &DbAddDel::on_idle_treeview_cell_edited_revert),
+               row, model_column_index));
           }
 
           do_change = false;
@@ -2448,6 +2460,26 @@ Application* DbAddDel::get_application()
   return dynamic_cast<Application*>(pWindow);
 }
 
+void DbAddDel::on_treeview_cell_editing_started(Gtk::CellEditable* cell_editable, const Glib::ustring& /* path */)
+{
+  //Start editing with previously-entered (but invalid) text,
+  //if we are allowing the user to correct some invalid data.
+  if(m_validation_retry)
+  {
+    //This is the CellEditable inside the CellRenderer.
+    Gtk::CellEditable* celleditable_validated = cell_editable;
+
+    //It's usually an Entry, at least for a CellRendererText:
+    Gtk::Entry* pEntry = dynamic_cast<Gtk::Entry*>(celleditable_validated);
+    if(pEntry)
+    {
+      pEntry->set_text(m_validation_invalid_text_for_retry);
+      m_validation_retry = false;
+      m_validation_invalid_text_for_retry.clear();
+    }
+  }
+}
+
 void DbAddDel::set_allow_view(bool val)
 {
   m_allow_view = val;
@@ -2558,6 +2590,7 @@ bool DbAddDel::start_new_record()
   else
   {
     std::cout << "start_new_record(): no editable rows." << std::endl;
+
     //The only keys are non-editable, so just add a row:
     select_item(iter); //without start_editing.
     //g_warning("start_new_record(): index_field_to_edit does not exist: %d", index_field_to_edit);
@@ -2570,7 +2603,7 @@ void DbAddDel::user_changed(const Gtk::TreeModel::iterator& row, guint col)
 {
   const Gnome::Gda::Value parent_primary_key_value = get_value_key(row);
   //std::cout << "DbAddDel::user_changed(): parent_primary_key_value=" << parent_primary_key_value.to_string() << std::endl;
- 
+
   sharedptr<const LayoutItem_Field> layout_field = get_column_field(col);
 
   if(!Conversions::value_is_empty(parent_primary_key_value)) //If the record's primary key is filled in:
@@ -2722,8 +2755,6 @@ void DbAddDel::user_changed(const Gtk::TreeModel::iterator& row, guint col)
 
 void DbAddDel::user_added(const Gtk::TreeModel::iterator& row)
 {
-  std::cout << "DEBUG: DbAddDel::user_added()" << std::endl;
-
   //Prevent impossible multiple related records:
   //The developer-mode UI now prevents the developer from using such a relationship anyway.
   if(m_allow_only_one_related_record && (get_count() > 0))
@@ -2757,14 +2788,14 @@ void DbAddDel::user_added(const Gtk::TreeModel::iterator& row)
     //This only works when the primary key is already stored: primary_key_value = get_value_key(row);
     //sharedptr<LayoutItem_Field> layout_item = sharedptr<LayoutItem_Field>::create();
     //layout_item->set_full_field_details(field);
-    
+
     primary_key_value = get_value_key_selected();
   }
 
   //If no primary key value is available yet, then don't add the record yet:
   if(Conversions::value_is_empty(primary_key_value))
     return;
-    
+
   #ifdef GLIBMM_EXCEPTIONS_ENABLED
   sharedptr<SharedConnection> sharedconnection = connect_to_server(get_application()); //Keep it alive while we need the data_model.
   #else
