@@ -909,8 +909,8 @@ void DbTreeModel::get_record_counts(gulong& total, gulong& found) const
       Glib::RefPtr<Gnome::Gda::SqlBuilder> builder =
         Gnome::Gda::SqlBuilder::create(Gnome::Gda::SQL_STATEMENT_SELECT);
 
-      const guint id_function = builder->add_function("count", builder->add_id("*")); //TODO: Is * allowed here?
-      builder->add_field_id(id_function);
+      const Gnome::Gda::SqlBuilder::Id id_function = builder->add_function("count", builder->add_id("*")); //TODO: Is * allowed here?
+      builder->add_field_value_id(id_function);
 
       builder->select_add_target(m_found_set.m_table_name);
 
@@ -926,7 +926,8 @@ void DbTreeModel::get_record_counts(gulong& total, gulong& found) const
           std::auto_ptr<Glib::Error> value_error;
           Gnome::Gda::Value value = datamodel->get_value_at(0, 0, value_error);
 #endif
-          total = (gulong)value.get_int64(); //I discovered that it's a int64 by trying it.
+	  // This will probably fail on Windows, where a long is only 32 bits wide.
+          total = static_cast<gulong>(value.get_int64()); //I discovered that it's a int64 by trying it.
         }
       }
     }
