@@ -43,10 +43,24 @@ public:
   virtual void remove(Gtk::Widget& first); //override
 
   void set_columns_count(guint value);
+  
+  guint get_columns_count() const;
 
-  /** Sets the padding to put between the child widgets.
+  /** Sets the padding to put between the columns of widgets.
    */
-  virtual void set_padding(guint padding);
+  void set_column_padding(guint padding);
+  
+  /** Gets the padding between the columns of widgets.
+   */
+  guint get_column_padding() const;
+  
+  /** Sets the padding to put between the rows of widgets.
+   */
+  void set_row_padding(guint padding);
+  
+  /** Gets the padding between the rows of widgets.
+   */
+  guint get_row_padding() const;
 
   /** Show extra UI that is useful in RAD tools:
    */
@@ -57,15 +71,16 @@ public:
   // Implement forall which is not implemented in gtkmm:
   typedef sigc::slot<void, Widget&> ForallSlot;
   void forall(const ForallSlot& slot);
+  
+  /** Get the column in which the specified "first" widget is placed.
+   * result false if the widget is not one of the "first" widgets, or 
+   * if has not yet been placed in a column, because the size has not yet been requested.
+   */
+  bool get_column_for_first_widget(const Gtk::Widget& first, guint& column);
 
-protected:
-#ifndef GLIBMM_VFUNCS_ENABLED
-  // These are the hand-coded C vfunc implementations in case the
-  // corresponding glibmm API has been disabled
-  static void glom_forall_impl(GtkContainer* container, gboolean include_internals, GtkCallback callback, gpointer callback_data);
-  static GType glom_child_type_impl(GtkContainer* container);
-#endif
-#ifndef GLIBMM_DEFAULT_SIGNAL_HANDLERS_ENABLED
+private:
+
+  #ifndef GLIBMM_DEFAULT_SIGNAL_HANDLERS_ENABLED
   // These are the hand-coded C default signal handlers in case the
   // corresponding glibmm API has been disabled
   static void glom_size_request_impl(GtkWidget* widget, GtkRequisition* requisition);
@@ -76,22 +91,27 @@ protected:
   static void glom_realize_impl(GtkWidget* widget);
   static void glom_unrealize_impl(GtkWidget* widget);
   static gboolean glom_expose_event_impl(GtkWidget* widget, GdkEventExpose* event);
-#endif
+  #endif
   //Overrides:
 
   //Handle child widgets:
   virtual void on_size_request(Gtk::Requisition* requisition);
-  virtual void on_size_allocate(Gtk::Allocation& allocation);
-  virtual GtkType child_type_vfunc() const;
+  virtual GType child_type_vfunc() const;
   virtual void on_add(Gtk::Widget* child);
   virtual void forall_vfunc(gboolean include_internals, GtkCallback callback, gpointer callback_data);
   virtual void on_remove(Gtk::Widget* child);
 
-  //Do extra drawing:
-  virtual void on_realize();
-  virtual void on_unrealize();
-  virtual bool on_expose_event(GdkEventExpose* event);
 
+protected:
+
+  virtual void on_size_allocate(Gtk::Allocation& allocation);
+  
+  //Do extra drawing:
+  //Virtual method overrides:
+  void on_realize();
+  void on_unrealize();
+  bool on_expose_event(GdkEventExpose* event);
+  
   int get_column_height(guint start_widget, guint widget_count, int& total_width) const;
 
   /** 
@@ -110,7 +130,11 @@ protected:
     bool m_expand_first_full;
     bool m_expand_second;
     
-    bool operator==(Gtk::Widget* child)
+    //The column that the widgets are currently in, due to the size/allocation.
+    bool m_has_allocated_column;
+    guint m_allocated_column;
+    
+    bool operator==(Gtk::Widget* child) const
     {
       return (child == m_first || child == m_second);
     }
@@ -120,7 +144,7 @@ protected:
     Gtk::Allocation m_second_allocation;
   };
 
-  typedef std::vector<FlowTableItem> type_vecChildren;
+private:
   void insert_before(FlowTableItem& item, Gtk::Widget& before);
 
   int get_item_requested_height(const FlowTableItem& item) const;
@@ -132,9 +156,16 @@ protected:
   Gtk::Allocation assign_child(Gtk::Widget* widget, int x, int y);
   Gtk::Allocation assign_child(Gtk::Widget* widget, int x, int y, int width, int height);
 
+protected:
+  typedef std::vector<FlowTableItem> type_vecChildren;
   type_vecChildren m_children;
+  
+  //Reset this and check it later to see if the layout of items has changed.
+  bool m_columns_allocated_changed;
+  
+private:
   guint m_columns_count;
-  guint m_padding;
+  guint m_column_padding, m_row_padding;
   bool m_design_mode;
 
   //Lines to draw in on_expose_event:
